# -*- coding: utf-8 -*-
"""
Created on Wed May  6 17:21:25 2020

@author: omedeiro
"""

import signal
import time
from sys import exit
from time import sleep

import numpy as np

import qnnpy.functions.functions as qf


class Snspd:
    """Class for SNSPD measurement. This class handels the instrument
    configuration, measurement, plotting, logging, and saving.
    See qnnpy\examples to see this class in action"""

    def __init__(self, configuration_file):
        #######################################################################
        #       Load .yaml configuraiton file
        #######################################################################

        # qf is a package of base functions that can be used in any class.
        self.properties = qf.load_config(configuration_file)

        self.sample_name = self.properties["Save File"]["sample name"]
        self.device_name = self.properties["Save File"]["device name"]
        self.device_type = self.properties["Save File"]["device type"]

        if self.properties.get("iv_sweep"):
            self.R_srs = self.properties["iv_sweep"]["series_resistance"]

        if self.properties.get("double_sweep"):
            self.R_srs_g = self.properties["double_sweep"]["series_resistance_g"]

        self.isw = 0
        self.instrument_list = []

        #######################################################################
        # Setup instruments
        #######################################################################

        self.inst = qf.Instruments(self.properties)
        # if there are any errors in setting up, compare older versions of
        # snspd on github with qf.Instruments
        signal.signal(signal.SIGINT, self.handler)

    def handler(self, signum, frame):
        """
        Safer exit handling (e.g. on ctrl+c)
        """
        try:
            self.inst.source.set_voltage(0)
            self.inst.source.set_output(False)
        except Exception:
            pass
        try:
            self.inst.awg.set_output(False)
        except Exception:
            pass
        exit(1)

    def get_property(self, property_type, prop):
        return self.properties[property_type][prop]

    def average_counts(self, counting_time, iterations, trigger_v):
        """

        This function sets the counter trigger voltage, integrates the number
        of counts over the counting time. The source is turned on before and turned off after.

        Average COUNT RATE per iteration is returned in [[ [Hz] ]]


        """

        count_rate_temp = []

        for i in range(iterations):
            self.inst.source.set_output(True)
            sleep(0.05)  # bias device
            voltage = self.inst.meter.read_voltage()
            sleep(0.05)
            self.inst.meter.local_key()
            # self.inst.source.set_voltage(np.sign(i)*1e-3); time.sleep(0.05); #reverse bias to unlatch?
            # self.inst.source.set_voltage(i*self.R_srs); time.sleep(0.05)
            try:
                self.inst.counter.set_impedance(self.properties["Counter"]["impedance"])
                self.inst.counter.set_coupling(self.properties["Counter"]["coupling"])
            except Exception:
                pass
            self.inst.counter.set_trigger(trigger_v)
            count_rate = self.inst.counter.count_rate(
                counting_time=counting_time
            )  # obtain counts
            count_rate_temp.append(count_rate)  # append counts for each itteration

            if voltage > 0.005:
                print(str(count_rate) + " wire switched")
            else:
                print(count_rate)

            self.inst.source.set_output(False)
            sleep(0.05)
        count_rate_avg = np.mean(count_rate_temp)  # average of each itteraton
        return count_rate_avg

    def tc_measurement(self, voltage, path):
        """
        Tc measurement for ICE Oxford. Path must include filename.txt
        Bias device at some value much lower than Ic.
        This code will bias device, measure voltage, turn off voltage, every
        two seconds. ICE will measure temp every 10 seconds. Slower sweep is
        better. Keyboard Interrupt to end. File will be saved at path.
        """
        self.inst.source.set_voltage(voltage)
        self.inst.source.set_output(True)
        while True:
            file = open(path, "a")
            if self.properties["Temperature"]["name"] == "ICE":
                temp = qf.ice_get_temp(select=1)
            else:
                temp = self.inst.temp.read_temp(self.inst.temp.channel)
            self.inst.source.set_output(True)
            sleep(1)
            voltage = self.inst.meter.read_voltage()
            line = str(temp) + ", " + str(voltage)
            print(line)
            file.write("\n" + line)
            file.close()
            self.inst.source.set_output(False)

            sleep(1)


###############################################################################
#   _____ __  ______  ________    ___   __________ ___________
#  / ___// / / / __ )/ ____/ /   /   | / ___/ ___// ____/ ___/
#  \__ \/ / / / __  / /   / /   / /| | \__ \\__ \/ __/  \__ \
# ___/ / /_/ / /_/ / /___/ /___/ ___ |___/ /__/ / /___ ___/ /
# /____/\____/_____/\____/_____/_/  |_/____/____/_____//____/
#
###############################################################################

# subclass inherits the properties of snspd


class TriggerSweep(Snspd):
    """Sweeps trigger on counter.

    Configuration : trigger_sweep
    [start: counter trigger start point],
    [stop: counter trigger stop point],
    [step: counter trigger step],
    [bias_voltage: device bias],
    [attenuation_db: optical attenuation],
    [counting_time: integration time on counter]
    """

    def run_sweep(self):
        """Sweep counter voltage start-stop

        (counter_trigger_voltage not used)

        """
        self.inst.meter.local_key()
        self.v_bias = self.properties["trigger_sweep"]["bias_voltage"]
        atten_level = self.properties["trigger_sweep"]["attenuation_db"]
        coupling_setting = self.properties["Counter"]["coupling"]
        impedance_setting = self.properties["Counter"]["impedance"]

        self.inst.source.set_output(False)
        sleep(0.2)
        self.inst.source.set_output(True)
        self.inst.source.set_voltage(0)
        sleep(0.2)
        self.inst.source.set_voltage(self.v_bias)
        sleep(1)
        self.inst.counter.set_impedance(impedance_setting)  # 50Ohm
        self.inst.counter.set_coupling(coupling_setting)  # 50Ohm

        counting_time = self.properties["trigger_sweep"]["counting_time"]
        start = self.properties["trigger_sweep"]["start"]
        stop = self.properties["trigger_sweep"]["stop"]
        step = self.properties["trigger_sweep"]["step"]
        count_rate = []
        atten_list = [atten_level, 100]
        atten_list_bool = [False, True]
        for i in range(2):
            print("\\\\\\\\ BEAM BLOCK: %s \\\\\\\\ " % atten_list_bool[i])
            self.inst.attenuator.set_attenuation_db(atten_list[i])
            self.inst.attenuator.set_beam_block(atten_list_bool[i])
            sleep(5)
            trigger_v, count_rate = self.inst.counter.scan_trigger_voltage(
                voltage_range=[start, stop],
                counting_time=counting_time,
                num_pts=int(np.floor((stop - start) / step)),
            )
            self.trigger_v = trigger_v
            if i == 0:
                self.counts_trigger_light = count_rate
            else:
                self.counts_trigger_dark = count_rate
            sleep(1)
        self.inst.attenuator.set_beam_block(True)

    def plot(self, close=True):
        ydata = [self.counts_trigger_dark, self.counts_trigger_light]
        label = [
            "Beam block: True",
            "Beam block: False",
        ]  # maybe update this to be the attenuation level

        full_path = qf.save(self.properties, "trigger_sweep")[0]
        print(full_path)
        qf.plot(
            self.trigger_v,
            ydata,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            xlabel="Trigger Voltage (V)",
            ylabel="Count Rate (Hz)",
            label=label,
            y_scale="log",
            path=full_path,
            close=close,
        )

    def save(self):
        data_dict = {
            "trigger_v": self.trigger_v,
            "counts_trigger_dark": self.counts_trigger_dark,
            "counts_trigger_light": self.counts_trigger_light,
            "v_bias": self.v_bias,
        }
        self.full_path = qf.save(
            self.properties,
            "trigger_sweep",
            data_dict,
            instrument_list=self.instrument_list,
        )


class TriggerSweepScope(Snspd):
    """Sweeps trigger on scope channel. Noise from counter was switing device.

    FIXME: loop through attenuation on and OFF

    This class uses the scope to output a voltage pulse at the AUX port on each
    scope trigger. The scope trigger is swept instead of the counter and the
    AUX output is connected to the counter.


    Configuration : trigger_sweep:
    [start: counter trigger start point],
    [stop: counter trigger stop point],
    [step: counter trigger step],
    [bias_voltage: device bias],
    [attenuation_db: optical attenuation],
    [counting_time: integration time on counter]
    [counter_trigger_voltage: trigger level set on counter]


    """

    def run_sweep(self):
        """Sweep scope trigger. counter trigger set with
        "counter_trigger_voltage"

        On Lecroy 760Zi: Utilities>Utilities Setup>Aux Output>Trigger Out> 1V into 1MOhm


        """
        self.v_bias = self.properties["trigger_sweep"]["bias_voltage"]
        atten_level = self.properties["trigger_sweep"]["attenuation_db"]

        self.inst.source.set_output(False)
        sleep(0.2)
        self.inst.source.set_output(True)
        self.inst.source.set_voltage(0)
        sleep(0.2)
        self.inst.source.set_voltage(self.v_bias)
        sleep(1)
        self.inst.counter.write(":INP:IMP 1E6")  # 1 MOhm input impedance
        ct_volt = self.properties["trigger_sweep"]["counter_trigger_voltage"]
        self.inst.counter.set_trigger(trigger_voltage=ct_volt)

        counting_time = self.properties["trigger_sweep"]["counting_time"]
        start = self.properties["trigger_sweep"]["start"]
        stop = self.properties["trigger_sweep"]["stop"]
        step = self.properties["trigger_sweep"]["step"]
        trigger_v = np.arange(start, stop, step)

        atten_list = [100, atten_level]
        atten_list_bool = [True, False]
        for a in range(2):
            print("\\\\\\\\ BEAM BLOCK: %s \\\\\\\\ " % atten_list_bool[a])
            self.inst.attenuator.set_attenuation_db(atten_list[a])
            self.inst.attenuator.set_beam_block(atten_list_bool[a])
            counts = []
            for i in trigger_v:
                self.inst.scope.set_trigger(
                    source=self.inst.scope_channel, volt_level=i, slope="positive"
                )
                tv, count_rate = self.inst.counter.scan_trigger_voltage(
                    voltage_range=[ct_volt, ct_volt],
                    counting_time=counting_time,
                    num_pts=1,
                )

                counts.append(count_rate)
                sleep(1)
                print("Scope Trigger: %0.3f" % (i))
            if a == 0:
                # this was added so that qf.plot can accept multiple ydata traces as lists. all single traces need to be arrays
                self.counts_trigger_dark = np.asarray(counts, dtype=np.float32)
            else:
                self.counts_trigger_light = np.asarray(counts, dtype=np.float32)
        self.inst.attenuator.set_beam_block(True)
        self.trigger_v = trigger_v

    def plot(self, close=True):
        ydata = [self.counts_trigger_dark, self.counts_trigger_light]
        label = [
            "Beam block: True",
            "Beam block: False",
        ]  # maybe update this to be the attenuation level

        full_path = qf.save(self.properties, "trigger_sweep")
        qf.plot(
            self.trigger_v,
            ydata,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            xlabel="Trigger Voltage (V)",
            ylabel="Count Rate (Hz)",
            label=label,
            y_scale="log",
            path=full_path,
            close=close,
        )

    def save(self):
        data_dict = {
            "trigger_v": self.trigger_v,
            "counts_trigger_dark": self.counts_trigger_dark,
            "counts_trigger_light": self.counts_trigger_light,
            "v_bias": self.v_bias,
        }
        self.full_path = qf.save(
            self.properties,
            "trigger_sweep",
            data_dict,
            instrument_list=self.instrument_list,
        )


class IvSweep(Snspd):
    """Class object for iv sweeps

    Configuration: iv_sweep:
    [start: initial bias current],
    [stop: final bias current],
    [steps: number of points],
    [sweep: number of itterations],
    [full_sweep: Include both positive and negative bias],
    [series_resistance: resistance at voltage source ie R_srs]

    """

    def run_sweep_fixed(self):
        """Runs IV sweep with config paramters
        This constructs #steps between start and 75% of final current.
        Then, #steps between 75% and 100% of final current.

        np.linspace()
        """
        # self.inst.source.reset()
        # self.inst.meter.reset()

        self.inst.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        steps = self.properties["iv_sweep"]["steps"]
        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop * 0.75, steps)  # Coarse
        Isource2 = np.linspace(stop * 0.75, stop, steps)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.inst.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        try:
            if self.inst.meter.name == "Keithley2400":
                self.inst.meter.setup_read_volt()
                self.inst.meter.write("OUTP ON")
        except Exception:
            pass

        for n in self.v_set:
            self.inst.source.set_voltage(n)
            sleep(0.1)  # CHANGE BACK TO 0.1

            vread = self.inst.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_sourceMeter(self):
        # self.inst.sourcemeter.reset()

        self.inst.sourcemeter.set_output(False)
        self.inst.sourcemeter.setup_2W_source_I_read_V()

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        steps = self.properties["iv_sweep"]["steps"]
        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop * 0.75, steps)  # Coarse
        Isource2 = np.linspace(stop * 0.75, stop, steps)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.I_set = np.tile(Isource, sweep)

        self.inst.sourcemeter.set_output(True)
        sleep(1)
        voltage = []
        current = []

        for n in self.I_set:
            self.inst.sourcemeter.set_current(n)
            sleep(0.1)  # CHANGE BACK TO 0.1

            vread = self.inst.sourcemeter.read_voltage()  # Voltage

            # iread = (n-vread)/self.R_srs#(set voltage - read voltage)

            # iread = self.inst.sourcemeter.read_current()
            iread = n

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_dynamic(self):
        """Runs IV sweep with config paramters
        This constructs [points_coarse] between start and [percent] of final current.
        Then, [points_fine] between [percent] and 100% of final current.

        uses np.linspace()
        """
        self.inst.source.reset()
        self.inst.meter.reset()

        self.inst.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        percent = self.properties["iv_sweep"]["percent"]
        num1 = self.properties["iv_sweep"]["points_coarse"]
        num2 = self.properties["iv_sweep"]["points_fine"]

        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop * percent, num1)  # Coarse
        Isource2 = np.linspace(stop * percent, stop, num2)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.inst.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        for n in self.v_set:
            self.inst.source.set_voltage(n)
            sleep(0.1)

            vread = self.inst.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def run_sweep_spacing(self):
        """Runs IV sweep with config paramters
        This constructs [points_coarse] between start and [percent] of final current.
        Then, [points_fine] between [percent] and 100% of final current.

        uses np.arange()
        """
        self.inst.source.reset()
        self.inst.meter.reset()

        self.inst.source.set_output(False)

        start = self.properties["iv_sweep"]["start"]
        stop = self.properties["iv_sweep"]["stop"]
        percent = self.properties["iv_sweep"]["percent"]
        spacing1 = self.properties["iv_sweep"]["spacing_coarse"]
        spacing2 = self.properties["iv_sweep"]["spacing_fine"]

        sweep = self.properties["iv_sweep"]["sweep"]
        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["iv_sweep"]["full_sweep"]
        Isource1 = np.arange(start, stop * percent + spacing1, spacing1)  # Coarse
        Isource2 = np.arange(stop * percent, stop + spacing2, spacing2)  # Fine

        if full_sweep:
            Isource = np.concatenate(
                [Isource1, Isource2, Isource2[::-1], Isource1[::-1]]
            )
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = np.concatenate([Isource1, Isource2])
        self.v_set = np.tile(Isource, sweep) * self.R_srs

        self.inst.source.set_output(True)
        sleep(1)
        voltage = []
        current = []

        for n in self.v_set:
            self.inst.source.set_voltage(n)
            sleep(0.1)

            vread = self.inst.meter.read_voltage()  # Voltage

            iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

            print("V=%.4f V, I=%.2f uA, R =%.2f" % (vread, iread * 1e6, vread / iread))
            voltage.append(vread)
            current.append(iread)

        self.v_read = voltage
        self.i_read = current

    def isw_calc(self):
        """Calculates critical current."""
        """
        Critical current is defined as the current one step before a voltage
        reading greater than 5mV.
        self.isw = self.i_read[np.argmax(np.array(self.v_read) > 0.005)-1]
        print('%.4f uA' % (self.isw*1e6))
        """

        """ New method looks at differential and takes average of points that 
        meet condition. Prints mean() and std().` std should be < 1-2µA
        """
        # isws = abs(self.i_read[np.where(abs(np.diff(self.i_read)/max(np.diff(self.i_read))) > 0.5)])
        # self.isw = self.i_read[np.argmax(np.array(self.v_read) > 0.005)-1]
        # print('%.4f uA' % (self.isw*1e6))

        try:
            switches = np.asarray(
                np.where(abs(np.diff(self.i_read) / max(np.diff(self.i_read))) > 0.8),
                dtype=int,
            ).squeeze()
            isws = abs(np.asarray([self.i_read[i] for i in switches]))
            print(isws)
            self.isw = isws.mean()
            print(
                "Isw_avg = %.4f µA :--: Isw_std = %.4f µA"
                % (self.isw * 1e6, isws.std() * 1e6)
            )
        except Exception:
            print("Could not calculate Isw. Isw set to 0")
            self.isw = 0

    def plot(self):
        full_path = qf.save(self.properties, "iv_sweep")
        qf.plot(
            np.array(self.v_read),
            np.array(self.i_read) * 1e6,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            xlabel="Voltage (V)",
            ylabel="Current (uA)",
            path=full_path,
            show=True,
            close=True,
        )

    def save(self, override_vals=None):
        # Set up data dictionary
        data_dict = {
            "V_source": self.v_set,
            "V_device": self.v_read,
            "I_device": self.i_read,
            "Isw": self.isw,
            "R_srs": self.R_srs,
            "temp": self.properties["Temperature"]["initial temp"],
        }

        self.full_path = qf.save(
            self.properties, "iv_sweep", data_dict, instrument_list=self.instrument_list
        )


class IvSweepScope(Snspd):
    def run_sweep(self):
        awg_amp = self.properties["iv_sweep_scope"]["awg_amp"]
        atten = self.properties["iv_sweep_scope"]["atten"]
        freq = self.properties["iv_sweep_scope"]["freq"]

        trigger_v = self.properties["iv_sweep_scope"]["trigger_v"]
        trigger_channel = self.properties["iv_sweep_scope"]["trigger_channel"]
        channels = self.properties["iv_sweep_scope"]["channels"]
        hist_channel = self.properties["iv_sweep_scope"]["hist_channel"]
        num_segments = self.properties["iv_sweep_scope"]["num_segments"]
        self.inst.awg.set_vpp(awg_amp)
        self.inst.awg.set_freq(freq)
        try:
            self.inst.awg.set_ramp(freq=freq, vpp=awg_amp, symm=0)
        except Exception:
            pass
        temperature = self.inst.temp.read_temp()
        # temperature=0
        self.inst.scope.set_trigger(source=trigger_channel, volt_level=trigger_v)
        self.inst.scope.pyvisa.timeout = 10000
        self.inst.scope.clear_sweeps()
        data = self.inst.scope.get_multiple_trace_sequence(
            channels=channels, NumSegments=num_segments
        )
        hist = self.inst.scope.get_wf_data(hist_channel)
        hist_dict = {hist_channel + "x": hist[0], hist_channel + "y": hist[1]}
        self.inst.scope.set_sample_mode()

        data.update(hist_dict)
        data.update(
            {"freq": freq, "awg_amp": awg_amp, "atten": atten, "temp": temperature}
        )

        try:
            self.inst.awg.set_output(False)
        except Exception:
            pass

        self.data_dict_iv_sweep_scope = data

    def save(self):
        self.full_path = qf.save(
            self.properties,
            "iv_sweep_scope",
            self.data_dict_iv_sweep_scope,
            instrument_list=self.instrument_list,
        )
        self.inst.scope.save_screenshot(
            self.full_path + "screen_shot" + ".png", white_background=False
        )


class DoubleSweep(Snspd):
    """Class object for iv sweeps

    Configuration: iv_sweep:
    [start: initial bias current],
    [stop: final bias current],
    [steps: number of points],
    [sweep: number of iterations],
    [full_sweep: Include both positive and negative bias],
    [series_resistance: resistance at voltage source ie R_srs]

    """

    def run_sweep(self):
        """Runs IV sweep with config paramters
        This constructs #steps between start and 75% of final current.
        Then, #steps between 75% and 100% of final current.


        """
        self.inst.source1.reset()
        self.inst.source2.reset()

        self.inst.meter.reset()

        self.inst.source1.set_output(False)
        self.inst.source2.set_output(False)

        start = self.properties["double_sweep"]["start"]
        stop = self.properties["double_sweep"]["stop"]
        steps = self.properties["double_sweep"]["steps"]
        sweep = self.properties["double_sweep"]["sweep"]
        Ig_start = self.properties["double_sweep"]["Ig_start"]
        Ig_stop = self.properties["double_sweep"]["Ig_stop"]
        Ig_steps = self.properties["double_sweep"]["Ig_steps"]

        # To select full (positive and negative) trace or half trace
        full_sweep = self.properties["double_sweep"]["full_sweep"]
        Isource1 = np.linspace(start, stop, steps)  # Coarse
        # Isource2 = np.linspace(stop*0.75, stop, steps) #Fine

        Isource_Ig = np.linspace(Ig_start, Ig_stop, Ig_steps)

        if full_sweep:
            Isource = np.concatenate([Isource1, Isource1[::-1]])
            Isource = np.concatenate([Isource, -Isource])
        else:
            Isource = Isource1
        self.v_set = np.tile(Isource, sweep) * self.R_srs
        self.v_set_g = np.tile(Isource_Ig, sweep) * self.R_srs_g

        self.inst.source.set_output(True)
        self.inst.source2.set_output(True)
        sleep(1)
        self.v_list = []
        self.i_list = []

        self.ig_list = []
        for m in self.v_set_g:
            voltage = []
            current = []
            self.inst.source2.set_voltage(m)
            sleep(0.1)
            for n in self.v_set:
                self.inst.source.set_voltage(n)
                sleep(0.1)

                vread = self.inst.meter.read_voltage()  # Voltage

                iread = (n - vread) / self.R_srs  # (set voltage - read voltage)

                print(
                    "Vd=%.4f V, Id=%.2f uA, Ig=%.2f uA, R =%.2f"
                    % (vread, iread * 1e6, m * 1e6 / self.R_srs_g, vread / iread)
                )
                voltage.append(vread)
                current.append(iread)

            # ADDED by Andrew...
            # Turn off both voltage sources to provide reset time
            self.inst.source.set_voltage(0.0)
            self.inst.source2.set_voltage(0.0)
            sleep(2)
            #######

            self.v_list.append(np.asarray(voltage, dtype=np.float32))
            self.i_list.append(np.asarray(current, dtype=np.float32))
            self.ig_list.append(m / self.R_srs_g)

    def plot(self):
        full_path = qf.save(self.properties, "double_sweep")
        qf.plot(
            self.v_list,
            self.i_list,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            xlabel="Voltage (V)",
            ylabel="Current (A)",
            label=self.ig_list,
            path=full_path,
            show=True,
            linestyle="-o",
            close=True,
        )

    def save(self, extra_values={}):
        # Set up data dictionary

        data_dict = {
            "V_source": self.v_set,
            "V_device": self.v_list,
            "I_device": self.i_list,
            "I_gate": self.ig_list,
            "R_srs": self.R_srs,
            "temp": self.properties["Temperature"]["initial temp"],
        }
        data_dict.update(extra_values)
        self.full_path = qf.save(
            self.properties,
            "double_sweep",
            data_dict,
            instrument_list=self.instrument_list,
        )


class PhotonCounts(Snspd):
    """Class object for PCR and DCR measurments. Configuration data will be
        used for both PCR and DCR

    Configuration: photon_counts:
    [start: initial bias current]
    [stop: final bias current]
    [step: current step size]
    [trigger_v: trigger voltage set on counter]
    [counting_time: integration time on counter]
    [iterations: number of repeated count measurements to average over]
    [attenuation_db: optical attenuation]
    """

    def __init__(self, config):
        """
        initialize optional variables as None type to enable successful
        plotting + saving without them
        """
        self.LCR = []
        self.DCR = []
        self.attenuation = 0
        Snspd.__init__(self, config)

    def dark_counts(self):
        """Dark Count Rate"""

        # Variable Setup
        iterations = self.properties["photon_counts"]["iterations"]
        start = self.properties["photon_counts"]["start"]
        stop = self.properties["photon_counts"]["stop"]
        step = self.properties["photon_counts"]["step"]
        counting_time = self.properties["photon_counts"]["counting_time"]
        trigger_v = self.properties["photon_counts"]["trigger_v"]

        currents = np.arange(start, stop, step)
        self.currents = currents

        # Block light
        self.inst.attenuator.set_attenuation_db(100)
        self.inst.attenuator.set_beam_block(True)

        count_rate_list = []

        print("\\\\\\\\ DARK COUNT RATE \\\\\\\\")
        for n, j in enumerate(currents):  # sweep current
            start_time = time.time()
            self.inst.source.set_voltage(voltage=j * self.R_srs)
            count_rate_avg = Snspd.average_counts(
                self, counting_time, iterations, trigger_v
            )

            count_rate_list.append(
                count_rate_avg
            )  # final countrate at this current is the average of each itteration
            print(
                "Current value %0.2f uA - Count rate = %0.2e   (%s of %s: %0.2f min)"
                % (
                    j * 1e6,
                    count_rate_avg,
                    n + 1,
                    len(currents),
                    (time.time() - start_time) / 60.0,
                )
            )

        self.inst.source.set_output(False)
        self.DCR = np.asarray(count_rate_list, dtype=np.float32)
        #        self.DCR = currents*2  # here for testing

        return self.DCR

    def light_counts(self):
        """Photon Count Rate"""
        # Variable Setup
        iterations = self.properties["photon_counts"]["iterations"]
        start = self.properties["photon_counts"]["start"]
        stop = self.properties["photon_counts"]["stop"]
        step = self.properties["photon_counts"]["step"]
        counting_time = self.properties["photon_counts"]["counting_time"]
        trigger_v = self.properties["photon_counts"]["trigger_v"]
        self.attenuation = self.properties["photon_counts"]["attenuation_db"]

        currents = np.arange(start, stop, step)
        self.currents = currents

        # Shine light
        self.inst.attenuator.set_attenuation_db(self.attenuation)
        self.inst.attenuator.set_beam_block(False)

        count_rate_list = []

        print("\\\\\\\\ LIGHT COUNT RATE \\\\\\\\")
        for n, j in enumerate(currents):  # sweep current
            start_time = time.time()
            self.inst.source.set_voltage(voltage=j * self.R_srs)
            sleep(0.1)
            count_rate_avg = Snspd.average_counts(
                self, counting_time, iterations, trigger_v
            )
            count_rate_list.append(
                count_rate_avg
            )  # final countrate at this current is the average of each itteration
            print(
                "Current value %0.2f uA - Count rate = %0.2e   (%s of %s: %0.2f min)"
                % (
                    j * 1e6,
                    count_rate_avg,
                    n + 1,
                    len(currents),
                    (time.time() - start_time) / 60.0,
                )
            )
        self.inst.attenuator.set_beam_block(True)
        self.inst.source.set_output(False)
        self.LCR = np.asarray(count_rate_list, dtype=np.float32)

        #        self.LCR = currents*3 # here for testing
        return self.LCR

    def dark_counts_noattn(self):
        """Dark Count Rate -- adjust laser manually"""
        # Variable Setup
        iterations = self.properties["photon_counts"]["iterations"]
        start = self.properties["photon_counts"]["start"]
        stop = self.properties["photon_counts"]["stop"]
        step = self.properties["photon_counts"]["step"]
        try:
            mid_start = self.properties["photon_counts"]["mid_start"]
            mid_stop = self.properties["photon_counts"]["mid_stop"]
            high_res_step = self.properties["photon_counts"]["high_res_step"]
            fine_sweep = True
        except Exception:
            fine_sweep = False
        counting_time = self.properties["photon_counts"]["counting_time"]
        trigger_v = self.properties["photon_counts"]["trigger_v"]

        if not fine_sweep:
            currents = np.arange(start, stop, step)
        else:
            currents = np.r_[
                start:mid_start:step,
                mid_start:mid_stop:high_res_step,
                mid_stop:stop:step,
            ]
        self.currents = currents

        count_rate_list = []

        print("\\\\\\\\ DARK COUNT RATE \\\\\\\\")
        for n, j in enumerate(currents):  # sweep current
            start_time = time.time()
            self.inst.source.set_voltage(voltage=j * self.R_srs)
            sleep(0.1)
            counts_at_current = []
            for i in range(self.properties["photon_counts"]["nonavg_iterations"]):
                count_rate_avg = Snspd.average_counts(
                    self, counting_time, iterations, trigger_v
                )
                counts_at_current.append(count_rate_avg)
            count_rate_list.append(
                counts_at_current
            )  # final countrate at this current is the average of each itteration
            print(
                "Current value %0.2f uA - Count rate = %0.2e   (%s of %s: %0.2f min)"
                % (
                    j * 1e6,
                    count_rate_avg,
                    n + 1,
                    len(currents),
                    (time.time() - start_time) / 60.0,
                )
            )

        self.DCR = np.asarray(count_rate_list, dtype=np.float32)
        self.inst.source.set_output(False)

        return self.DCR

    def light_counts_noattn(self):
        """Photon Count Rate -- adjust laser manually"""
        # Variable Setup
        iterations = self.properties["photon_counts"]["iterations"]
        start = self.properties["photon_counts"]["start"]
        stop = self.properties["photon_counts"]["stop"]
        step = self.properties["photon_counts"]["step"]
        try:
            mid_start = self.properties["photon_counts"]["mid_start"]
            mid_stop = self.properties["photon_counts"]["mid_stop"]
            high_res_step = self.properties["photon_counts"]["high_res_step"]
            fine_sweep = True
        except Exception:
            fine_sweep = False
        counting_time = self.properties["photon_counts"]["counting_time"]
        trigger_v = self.properties["photon_counts"]["trigger_v"]
        self.attenuation = self.properties["photon_counts"]["attenuation_db"]

        if not fine_sweep:
            currents = np.arange(start, stop, step)
        else:
            currents = np.r_[
                start:mid_start:step,
                mid_start:mid_stop:high_res_step,
                mid_stop:stop:step,
            ]
        self.currents = currents

        count_rate_list = []

        print("\\\\\\\\ LIGHT COUNT RATE \\\\\\\\")
        for n, j in enumerate(currents):  # sweep current
            start_time = time.time()
            self.inst.source.set_voltage(voltage=j * self.R_srs)
            sleep(0.1)
            counts_at_current = []
            for i in range(self.properties["photon_counts"]["nonavg_iterations"]):
                count_rate_avg = Snspd.average_counts(
                    self, counting_time, iterations, trigger_v
                )
                counts_at_current.append(count_rate_avg)
            count_rate_list.append(
                counts_at_current
            )  # final countrate at this current is the average of each itteration
            print(
                "Current value %0.2f uA - Count rate = %0.2e   (%s of %s: %0.2f min)"
                % (
                    j * 1e6,
                    count_rate_avg,
                    n + 1,
                    len(currents),
                    (time.time() - start_time) / 60.0,
                )
            )
        self.LCR = np.asarray(count_rate_list, dtype=np.float32)

        return self.LCR

    def plot(self):
        """Plots DCR, LCR, and LCR-DCR"""
        full_path = qf.save(self.properties, "photon_counts")[0]

        ydata = []
        label = []
        if self.DCR is not None and self.DCR != []:
            ydata.append(self.DCR)
            label.append("DCR")
        if self.LCR is not None and self.LCR != []:
            ydata.append(self.LCR)
            label.append("LCR")
        if (
            self.DCR is not None
            and self.LCR is not None
            and self.DCR != []
            and self.LCR != []
        ):
            ydata.append(self.LCR - self.DCR)
            label.append("LCR-DCR")
        if self.properties["Laser"]["wavelength_nm"]:
            device_name = (
                self.device_name
                + " "
                + str(self.properties["Laser"]["wavelength_nm"])
                + "nm"
                + " "
                + str(self.properties["photon_counts"]["attenuation_db"])
                + "dB"
            )
        qf.plot(
            self.currents * 1e6,
            ydata,
            title=self.sample_name + " " + self.device_type + " " + device_name,
            xlabel="Current (uA)",
            ylabel="Count Rate (Hz)",
            label=label,
            path=full_path,
            show=True,
            close=True,
        )

    def save(self):
        """If LCR was not measured, attunation is 100dB for dark counts"""
        if self.LCR is None or self.LCR == []:
            self.attenuation = 100

        self.instrument_list.append("Laser")

        data_dict = {
            "LCR": self.LCR,
            "DCR": self.DCR,
            "currents": self.currents,
            "wavelength": self.properties["Laser"]["wavelength_nm"],
            "attenuation": self.attenuation,
        }

        full_path = qf.save(
            self.properties,
            "photon_counts",
            data_dict,
            instrument_list=self.instrument_list,
        )


class LinearityCheck(Snspd):
    """Class object for Linearity Measurement.
    Sweeps atteunation and measures count rate.
    Assumes R_srs has already been defined.

    Configuration: linearity_check:
    [bias: current bias]
    [start: initial optical attenuation in dB]
    [stop: final optical attenuation in dB]
    [step: step size of optical attenuation in dB]
    [counting_time: integration time on counter]
    [iterations: number of repeated count measurements to average over]

    FIXME: Trigger voltage on counter. Passed from self?
    """

    def run_sweep(self):
        bias = self.properties["linearity_check"]["bias"]
        start = self.properties["linearity_check"]["start"]
        stop = self.properties["linearity_check"]["stop"]
        step = self.properties["linearity_check"]["step"]
        trigger_v = self.properties["linearity_check"]["trigger_v"]
        counting_time = self.properties["linearity_check"]["counting_time"]
        iterations = self.properties["linearity_check"]["iterations"]

        dbs = np.arange(start, stop, step)
        self.inst.source.set_output(True)
        self.inst.source.set_voltage(voltage=bias * self.R_srs)

        # add current loop if desired
        self.inst.attenuator.set_beam_block(False)

        counts_per_atten = []
        start_time = time.time()
        for i in dbs:
            self.inst.attenuator.set_attenuation_db(i)
            sleep(0.1)
            count_rate_avg = Snspd.average_counts(
                self, counting_time, iterations, trigger_v
            )
            counts_per_atten.append(count_rate_avg)
            print(
                "Attenuation: %.1f dB \\\\ Counts: %.0f \\\\ Elapsed Time: %.2f"
                % (i, count_rate_avg, time.time() - start_time)
            )
        self.inst.attenuator.set_beam_block(True)

        self.counts_per_atten = np.asarray(counts_per_atten, dtype=np.float32)
        self.attenuation_levels = np.asarray(dbs, dtype=np.float32)
        self.linearity_bias = bias

    def run_manual_sweep(self):
        bias = self.properties["linearity_check"]["bias"]
        trigger_v = self.properties["linearity_check"]["trigger_v"]
        counting_time = self.properties["linearity_check"]["counting_time"]
        iterations = self.properties["linearity_check"]["iterations"]

        self.inst.source.set_output(True)
        self.inst.source.set_voltage(voltage=bias * self.R_srs)

        continue_sweep = True
        counts_per_atten = []
        dbs = []
        start_time = time.time()
        while continue_sweep:
            sleep(0.1)
            db = float(input("Attenuation?"))
            count_rate_avg = Snspd.average_counts(
                self, counting_time, iterations, trigger_v
            )
            counts_per_atten.append(count_rate_avg)
            print(
                "Attenuation: %.1f dB \\\\ Counts: %.0f \\\\ Elapsed Time: %.2f"
                % (db, count_rate_avg, time.time() - start_time)
            )
            dbs.append(db)
            continue_sweep = int(input("Continue sweep?"))
            print(continue_sweep)
        self.counts_per_atten = np.asarray(counts_per_atten, dtype=np.float32)
        self.attenuation_levels = np.asarray(dbs, dtype=np.float32)
        self.linearity_bias = bias

    def plot(self):
        full_path = qf.save(self.properties, "linearity_check")
        device_name = (
            self.device_name
            + " "
            + str(self.properties["Laser"]["wavelength_nm"])
            + "nm"
        )
        qf.plot(
            self.attenuation_levels,
            self.counts_per_atten,
            title=self.sample_name + " " + self.device_type + " " + device_name,
            xlabel="Attenuation (dB)",
            ylabel="Count Rate (Hz)",
            y_scale="log",
            path=full_path,
            show=True,
            close=True,
        )

    def save(self):
        data_dict = {
            "attenuation": self.attenuation_levels,
            "counts": self.counts_per_atten,
            "bias": self.linearity_bias,
        }

        self.full_path = qf.save(
            self.properties,
            "linearity_check",
            data_dict,
            instrument_list=self.instrument_list,
        )


class PulseTraceSingle(Snspd):
    """Single trace acquistion for LeCroy620Zi.
    no pulse_trace configuration required.

    Sadly, this will reset instruments when initializing the Snspd class.
    So be sure to 'stop' the scope before running.
    """

    def trace_data(self, channels=None, trigger_source=None):
        """Returns x,y of scope_channel in configuration file"""

        bias = self.properties["pulse_trace"]["bias_voltage"]
        self.inst.scope.set_trigger_mode(trigger_mode="Stop")
        self.inst.source.set_output(True)
        self.inst.source.set_voltage(voltage=bias)

        if channels:
            channels = channels
        else:
            channels = self.properties["pulse_trace"]["channel"]

        xlist = []
        ylist = []
        tlist = []

        trigger_v = self.properties["pulse_trace"]["trigger_level"]
        if trigger_source:
            self.inst.scope.set_trigger(
                source=trigger_source, volt_level=trigger_v, slope="positive"
            )
        else:
            self.inst.scope.set_trigger(
                source=channels[0], volt_level=trigger_v, slope="positive"
            )

        if self.properties.get("Attenuator"):
            attenuation = self.properties["pulse_trace"]["attenuation"]
            self.inst.attenuator.set_attenuation_db(attenuation)
            if attenuation == 100:
                self.inst.attenuator.set_beam_block(True)
            else:
                self.inst.attenuator.set_beam_block(False)

        sleep(0.1)
        self.inst.scope.set_trigger_mode(trigger_mode="Single")
        while self.inst.scope.get_trigger_mode() == "Single\n":
            sleep(0.1)

        for i in range(len(channels)):
            x, y = self.inst.scope.get_single_trace(channel=channels[i])
            xlist.append(x)
            ylist.append(y)

        self.trace_x = xlist
        self.trace_y = ylist
        self.inst.source.set_output(False)
        return self.trace_x, self.trace_y

    def plot(self):
        """Grabs new trace from scope and plots. Figure is saved to path"""
        # x,y = self.inst.scope.get_single_trace(channel= self.inst.scope_channel)
        channels = self.properties["pulse_trace"]["channel"]
        full_path, time_str = qf.save(self.properties, "pulse_trace")
        #        data_dict = {'x':x,'y':y}
        x = self.trace_x
        y = self.trace_y
        qf.plot(
            x,
            y,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            # xlabel = '',
            # ylabel = '',
            path=full_path,
            show=True,
            linestyle="-",
            label=channels,
            close=True,
        )

    def save(self):
        data_dict = {"x": self.trace_x, "y": self.trace_y}
        file_path, time_str = qf.save(
            self.properties,
            "pulse_trace",
            data_dict,
            instrument_list=self.instrument_list,
        )
        self.inst.scope.save_screenshot(file_path + ".png")


class PulseTraceMultiple(Snspd):
    def trace_data(self, channels=None, trigger_source=None):
        """Returns x,y of scope_channel in configuration file"""

        if channels:
            channels = channels
        else:
            channels = self.properties["pulse_trace"]["channel"]

        """ Option for attenuator control. Set to 100dB for beam block. """
        if self.properties.get("pulse_trace").get("attenuation"):
            self.inst.attenuator.set_attenuation_db(
                self.properties["pulse_trace"]["attenuation"]
            )
            self.inst.attenuator.set_beam_block(False)
            if self.properties["pulse_trace"]["attenuation"] == 100:
                self.inst.attenuator.set_beam_block(True)

        total_xlist = []
        total_ylist = []
        trigger_v = self.properties["pulse_trace"]["trigger_level"]
        number_of_traces = self.properties["pulse_trace"]["number_of_traces"]

        if trigger_source:
            self.inst.scope.set_trigger(
                source=trigger_source, volt_level=trigger_v, slope="positive"
            )
        else:
            self.inst.scope.set_trigger(
                source=channels[0], volt_level=trigger_v, slope="positive"
            )

        self.inst.source.set_voltage(self.properties["pulse_trace"]["bias_voltage"])
        self.inst.source.set_output(True)
        for n in range(number_of_traces):
            self.inst.meter.read_voltage()
            self.inst.scope.set_trigger_mode(trigger_mode="Single")
            while self.inst.scope.get_trigger_mode() != "Stopped\n":
                sleep(0.001)

            xlist = []
            ylist = []
            for i in range(len(channels)):
                x, y = self.inst.scope.get_single_trace(channel=channels[i])
                xlist.append(x)  # keep all x data the same.
                ylist.append(y)

            total_xlist.append(np.asarray(xlist, dtype=np.float32))
            total_ylist.append(np.asarray(ylist, dtype=np.float32))

            print("Acquired Trace %0.f of %0.f" % (n + 1, number_of_traces))

        self.trace_x = np.asarray(total_xlist, dtype=np.float32)
        self.trace_y = np.asarray(total_ylist, dtype=np.float32)
        self.inst.source.set_output(False)

        return self.trace_x, self.trace_y

    def plot(self):
        """Grabs new trace from scope and plots. Figure is saved to path"""
        # x,y = self.inst.scope.get_single_trace(channel= self.inst.scope_channel)

        full_path = qf.save(self.properties, "pulse_trace")
        #        data_dict = {'x':x,'y':y}
        qf.plot(
            self.trace_x,
            self.trace_y,
            title=self.sample_name + " " + self.device_type + " " + self.device_name,
            # xlabel = '',
            # ylabel = '',
            path=full_path,
            show=True,
            linestyle="-",
            close=True,
        )

    def save(self):
        data_dict = {"x": self.trace_x, "y": self.trace_y}
        qf.save(
            self.properties,
            "pulse_trace",
            data_dict,
            instrument_list=self.instrument_list,
        )


class PulseTraceSegments(Snspd):
    def trace_data(self, channels=None, trigger_source=None, bias_voltage=None):
        """Returns x,y of scope_channel in configuration file"""

        if channels:
            channels = channels
        else:
            channels = self.properties["pulse_trace"]["channel"]

        """ Option for attenuator control. Set to 100dB for beam block. """
        if self.properties.get("Attenuator"):
            attenuation = self.properties["pulse_trace"]["attenuation"]
            self.inst.attenuator.set_attenuation_db(attenuation)
            if attenuation == 100:
                self.inst.attenuator.set_beam_block(True)
            else:
                self.inst.attenuator.set_beam_block(False)

        total_ylist = []
        trigger_v = self.properties["pulse_trace"]["trigger_level"]
        number_of_traces = self.properties["pulse_trace"]["number_of_traces"]

        if trigger_source:
            self.inst.scope.set_trigger(
                source=trigger_source, volt_level=trigger_v, slope="positive"
            )
        else:
            self.inst.scope.set_trigger(
                source=channels[0], volt_level=trigger_v, slope="positive"
            )

        if bias_voltage:
            self.inst.source.set_voltage(bias_voltage)
        else:
            self.inst.source.set_voltage(self.properties["pulse_trace"]["bias_voltage"])
        self.inst.source.set_output(True)

        if trigger_source:
            self.inst.scope.set_trigger(
                source=trigger_source, volt_level=trigger_v, slope="positive"
            )
        else:
            self.inst.scope.set_trigger(
                source=channels[0], volt_level=trigger_v, slope="positive"
            )

        self.inst.scope.pyvisa.timeout = 10000
        self.inst.scope.clear_sweeps()
        data = self.inst.scope.get_multiple_trace_sequence(
            channels=channels, NumSegments=number_of_traces
        )

        self.inst.scope_data = data
        self.inst.scope.set_sample_mode()
        self.inst.scope.set_trigger_mode()
        self.inst.source.set_output(False)

        return self.inst.scope_data

    def plot(self):
        """Grabs new trace from scope and plots. Figure is saved to path"""
        # x,y = self.inst.scope.get_single_trace(channel= self.inst.scope_channel)
        full_path = qf.save(self.properties, "pulse_trace")
        self.inst.scope.set_trigger_mode("Single")
        self.inst.scope.save_screenshot(full_path + ".png")
        self.inst.scope.set_trigger_mode()

    # #        data_dict = {'x':x,'y':y}
    #         qf.plot(self.trace_x, self.trace_y,
    #                 title=self.sample_name+" "+self.device_type+" "+
    #                       self.device_name,
    #                 # xlabel = '',
    #                 # ylabel = '',
    #                 path=full_path,
    #                 show=True,
    #                 linestyle='-',
    #                 close=True)

    def save(self, data=None):
        data_dict = {
            "data": self.inst.scope_data,
            "atten": self.properties["pulse_trace"]["attenuation"],
            "vbias": self.properties["pulse_trace"]["bias_voltage"],
        }

        if data:
            data_dict.update(data)
        qf.save(
            self.properties,
            "pulse_trace",
            data_dict,
            instrument_list=self.instrument_list,
        )


class PulseTraceCurrentSweep(Snspd):
    """UNFINISHED Sweep current and acquire trace. Configuration requires
    pulse_trace section: {start, stop, steps}"""

    def trace_data(self):
        start = self.properties["pulse_trace"]["start"]
        stop = self.properties["pulse_trace"]["stop"]
        steps = self.properties["pulse_trace"]["steps"]
        num_traces = 1
        currents = np.linspace(start, stop, steps)

        snspd_traces_x_list = []
        snspd_traces_y_list = []
        pd_traces_x_list = []
        pd_traces_y_list = []
        start_time = time.time()
        self.inst.source.set_output(True)
        for n, i in enumerate(currents):
            print(
                "   ---   Time elapsed for measurement %s of %s: %0.2f "
                "min    ---   " % (n, len(currents), (time.time() - start_time) / 60.0)
            )
            self.inst.source.set_voltage(i * self.R_srs)
            pd_traces_x = []  # Photodiode pulses
            pd_traces_y = []
            snspd_traces_x = []  # Device pulses
            snspd_traces_y = []
            self.inst.scope.clear_sweeps()
            for n in range(num_traces):
                x, y = self.inst.scope.get_single_trace(channel="C2")
                snspd_traces_x.append(x)
                snspd_traces_y.append(y)
                x, y = self.inst.scope.get_single_trace(channel="C3")
                pd_traces_x.append(x)
                pd_traces_y.append(y)

            snspd_traces_x_list.append(snspd_traces_x)
            snspd_traces_y_list.append(snspd_traces_y)
            pd_traces_x_list.append(pd_traces_x)
            pd_traces_y_list.append(pd_traces_y)


class KineticInductancePhase(Snspd):
    def run_sweep_current(self, Ic_break=None, save=None, plot=None):
        self.inst.VNA.write("CALC:FORM SMITh")
        start = self.properties["kinetic_inductance_phase"]["start"]
        stop = self.properties["kinetic_inductance_phase"]["stop"]
        steps = self.properties["kinetic_inductance_phase"]["steps"]

        currents = np.linspace(start, stop, steps)

        self.if_bandwidth = self.inst.VNA.get_if_bw()
        self.fspan = self.inst.VNA.get_span()
        sweep_time = self.fspan / (self.if_bandwidth**2)

        self.rf_power = self.inst.VNA.get_power()

        self.inst.VNA.select_measurement()

        for current in currents:
            print("Bias set to %0.3f uA" % (current * 1e6))
            self.inst.source.set_voltage(current * self.R_srs)
            self.inst.source.set_output(True)
            self.current = current * 1e6

            if self.properties["Temperature"]["name"] == "ICE":
                self.currentTemp = qf.ice_get_temp(select=1)
            else:
                self.currentTemp = self.inst.temp.read_temp(self.inst.temp.channel)

            self.voltage = self.inst.meter.read_voltage()
            if Ic_break:
                if self.voltage > 0.01:
                    print("Wire Switched")
                    self.inst.meter.reset()
                    break
            t = sweep_time + 10
            # print('Waiting %0.1f seconds for vna averging' % t)
            print("Waiting 10 seconds for vna averging")

            sleep(10)  # vna averging

            self.f, self.re, self.im = self.inst.VNA.get_freq_real_imag()
            self.log_mag = 20 * np.log10(
                np.sqrt(np.square(self.re) + np.square(self.im))
            )
            self.phase = np.angle(self.re + self.im * 1j, deg=True)

            if plot:
                self.plot()
            if save:
                self.save()
        self.inst.source.set_output(False)
        self.inst.VNA.write("CALC:FORM MLOG")

    def plot(self):
        full_path = qf.save(self.properties, "kinetic_inductance_phase")
        qf.plot(
            self.f,
            self.phase,
            title=self.sample_name
            + " "
            + self.device_type
            + " "
            + self.device_name
            + ": "
            + str(round(self.current, 2))
            + " uA, "
            + str(self.rf_power)
            + " dB",
            xlabel="Frequency (Hz)",
            ylabel="Phase (°)",
            path=full_path,
            show=True,
            linestyle="-",
            close=True,
        )

    def save(self):
        data_dict = {
            "freq": self.f,
            "re": self.re,
            "im": self.im,
            "log_mag": self.log_mag,
            "phase": self.phase,
            "Ibias": self.current,
            "voltage": self.voltage,
            "if_bandwidth": self.if_bandwidth,
            "R_srs": self.R_srs,
            "temp": self.currentTemp,
        }
        qf.save(
            self.properties,
            "kinetic_inductance_phase",
            data_dict,
            instrument_list=self.instrument_list,
        )


class PulseTraceFFT_CurrentSweep(Snspd):
    """((NO PLOT. ONLY SAVE)) Sweep current and acquire traces C1(signal) and F1(FFT) . Configuration requires
    pulse_trace section: {start, stop, steps}"""

    def trace_data(self):
        start = self.properties["pulse_trace"]["start"]
        stop = self.properties["pulse_trace"]["stop"]
        steps = self.properties["pulse_trace"]["steps"]

        num_traces = 1
        currents = np.linspace(start, stop, steps)

        self.FFT = []
        self.FREQ = []
        self.VIN = []
        self.T1 = []
        self.VOUT = []
        start_time = time.time()
        self.inst.source.set_output(True)

        self.inst.scope.clear_sweeps()
        time.sleep(1)

        for n, i in enumerate(currents):
            print(
                "   ---   Time elapsed for measurement %s of %s: %0.2f "
                "min    ---   " % (n, len(currents), (time.time() - start_time) / 60.0)
            )
            self.inst.source.set_voltage(i * self.R_srs)

            time.sleep(1)

            self.inst.scope.label_channel("C1", "V_device")

            self.inst.scope.set_math("F1", "FFT", "C1")
            self.inst.scope.view_channel("F1")

            t1, C1 = self.inst.scope.get_wf_data("C1")
            tf, Ft = self.inst.scope.get_wf_data("F1")
            self.FFT.append(Ft)
            self.FREQ.append(tf)
            self.T1.append(t1)
            self.VOUT.append(C1)
            self.Ibias = currents

            time.sleep(1)

        self.inst.source.set_voltage(0)
        self.inst.source.set_output(False)

    def save(self):
        data_dict = {
            "FFT": self.FFT,
            "FREQ": self.FREQ,
            "T2": self.T1,
            "VOUT": self.VOUT,
            "Ibias": self.Ibias,
        }

        qf.save(
            self.properties,
            "pulse_trace",
            data_dict,
            instrument_list=self.instrument_list,
        )



import datetime
import time
from time import sleep

import numpy as np
import pyvisa


class LeCroy620Zi(object):
    """Python class for LeCroy Oscilloscope, written by Adam McCaughan.  Most of these commands
    originate from the Automation Command Reference Manual for WaveRunner Oscilloscopes"""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name, write_termination="\n")
        self.pyvisa.timeout = 10000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands
        self.write("COMM_HEADER OFF")  # Get rid of the leading 'VBS ' crap
        self.write(
            "COMM_FORMAT DEF9,WORD,BIN"
        )  # Set output to 16 bits of information (a 'word') per datapoint

    def read(self):
        return self.pyvisa.read()

    def read_raw(self):
        return self.pyvisa.read_raw()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def round_up_lockstep(self, x):
        """Some functions on the LeCroy require numbers to be rounded up to nearest 1,2 or 5
        e.g. 1.2e-6 -> 5e-6 and 4.7e0 -> 5e0"""
        x_str = "%0.9e" % x  # Takes 1234.24e-10 -> '1.234240000e-07'
        x_digits = x_str[:-4]  #  Takes '1.234240000e-07' -> '1.234240000'
        x_exp = x_str[-4:]
        if float(x_digits) <= 1:
            locked_x = "1"
        elif float(x_digits) <= 2:
            locked_x = "2"
        elif float(x_digits) <= 5:
            locked_x = "5"
        else:
            locked_x = "10"
        return float(locked_x + x_exp)

    def vbs_ask(self, message):
        vbs_msg = "VBS? 'return = %s'" % message
        # print 'Sending command:  ' + vbs_msg
        return self.query(vbs_msg)

    def vbs_write(self, message):
        vbs_msg = "VBS '%s'" % message
        # print 'Sending command:  ' + vbs_msg
        self.write(vbs_msg)

    def reset(self):
        self.write("*RST")
        self.write("COMM_HEADER OFF")  # Get rid of the leading 'VBS ' crap
        self.write(
            "COMM_FORMAT DEF9,WORD,BIN"
        )  # Set output to 16 bits of information (a word) per sample
        time.sleep(1)

    def clear_sweeps(self):
        self.vbs_write("app.ClearSweeps")  #
        time.sleep(0.2)  # Necessary to allow the scope time to reset all values

    def wait_until_idle(self, max_time=5):
        self.vbs_write(f"app.WaitUntilIdle({max_time})")

    def set_dialog_page(self, page="Measure"):
        # changes the displayed dialog page
        self.vbs_write('app.SystemControl.DialogPage = "%s"' % (page))

    def set_dialog_page_right(self, page="Skew Clock 1"):
        # changes the displayed dialog page
        self.vbs_write('app.SystemControl.RightDialogPage = "%s"' % (page))

    def view_channel(self, channel="C1", view=True):
        if channel[0] == "C":  # If it's C1, C3, etc
            self.vbs_write("app.Acquisition.%s.View = %s" % (channel, view))
        elif channel[0] == "F":  # If it's F1, F2...
            self.vbs_write("app.Math.%s.View = %s" % (channel, view))

    def set_active_input(self, channel="C1", activeInput="InputA"):
        self.vbs_write('app.Acquisition.%s.ActiveInput = "%s"' % (channel, activeInput))

    def set_coupling(self, channel="C1", coupling="DC1M"):
        """Coupling should be either AC1M, DC1M, DC50, or Gnd"""
        self.vbs_write('app.Acquisition.%s.Coupling = "%s"' % (channel, coupling))

    def get_coupling(self, channel="C1"):
        """Coupling should be either AC1M, DC1M, DC50, or Gnd"""
        return self.vbs_ask("app.Acquisition.%s.Coupling" % (channel))

    def set_bandwidth(self, channel="C1", bandwidth="Full"):
        """Bandwidth should be either 1GHz, 200MHz, 20MHz, 3GHz, 4GHz, Full"""
        self.vbs_write(
            'app.Acquisition.%s.BandwidthLimit = "%s"' % (channel, bandwidth)
        )

    def set_vertical_scale(self, channel="C1", volts_per_div=1, volt_offset=0):
        # Lecroy only allows digits 1, 2, and 5.  e.g. 5e-6 is acceptable, 4e-6 is not
        volts_per_div = self.round_up_lockstep(volts_per_div)
        self.vbs_write("app.Acquisition.%s.VerScale = %0.0e" % (channel, volts_per_div))
        self.vbs_write("app.Acquisition.%s.VerOffset = %0.0e" % (channel, volt_offset))

    def find_vertical_scale(self, channel="C1"):
        self.vbs_write("app.Acquisition.%s.FindScale" % channel)

    def set_horizontal_scale(self, time_per_div=1e-6, time_offset=0):
        self.vbs_write("app.Acquisition.Horizontal.HorScale = %0.6e" % time_per_div)
        self.vbs_write("app.Acquisition.Horizontal.HorOffset = %0.6e" % time_offset)

    def set_max_samples(self, num_samples=1e6):
        self.vbs_write("app.Acquisition.Horizontal.MaxSamples = %0.3e" % num_samples)

    def set_sample_rate(self, num_samples=1e9):
        self.vbs_write("app.Acquisition.Horizontal.SampleRate = %0.3e" % num_samples)

    def set_num_points(self, num_samples=1e3):
        self.vbs_write("app.Acquisition.Horizontal.NumPoints = %0.3e" % num_samples)

    def set_trigger(self, source="C1", volt_level=0.1, slope="positive"):
        """Slope should be "Either" / "Negative" / "Positive" """
        self.vbs_write('app.Acquisition.Trigger.Source = "%s"' % source)
        self.vbs_write(
            "app.Acquisition.Trigger.%s.Level = %0.4e" % (source, volt_level)
        )
        self.vbs_write('app.Acquisition.Trigger.%s.Slope = "%s"' % (source, slope))

    def set_trigger_mode(self, trigger_mode="Normal"):
        """trigger_mode should be set to Auto/Normal/Single/Stop"""
        self.vbs_write('app.Acquisition.TriggerMode = "%s"' % trigger_mode)

    def set_sample_mode(self, sample_mode="RealTime"):
        """Set Sample mode to either RealTime, Sequence, WStream"""
        return self.vbs_write(
            'app.Acquisition.Horizontal.SampleMode = "%s"' % sample_mode
        )

    def set_sequence_mode(self):
        return self.vbs_write('app.Acquisition.Horizontal.SampleMode = "Sequence"')

    def set_segments(self, NumSegments=500):
        return self.vbs_write(
            "app.Acquisition.Horizontal.NumSegments = %d" % NumSegments
        )

    def get_number_points_hor(self):
        return np.int(self.vbs_ask("app.Acquisition.Horizontal.NumPoints"))

    def get_time_point(self):
        return np.float(self.vbs_ask("app.Acquisition.Horizontal.TimePerPoint"))

    def set_persistence(self, channel="C1", persistence=False, monochrome=False):
        self.vbs_write('app.Display.LockPersistence = "PerTrace"')
        self.vbs_write("app.Acquisition.%s.Persisted = %s" % (channel, persistence))
        self.vbs_write(
            "app.Acquisition.%s.PersistenceMonochrome = %s" % (channel, monochrome)
        )

    def label_channel(self, channel="C1", label="Channel 1 label text"):
        if (label == "") or (label is False) or (label is None):
            self.vbs_write("app.Acquisition.%s.ViewLabels = False" % channel)
        else:
            self.vbs_write('app.Acquisition.%s.LabelsText = "%s"' % (channel, label))
            self.vbs_write("app.Acquisition.%s.ViewLabels = True" % channel)

    def set_display_gridmode(self, gridmode="Auto"):
        """gridmode should be Auto / Dual / Octal / Quad / Single / XY / XYDual / XYSingle"""
        self.vbs_write('app.Display.GridMode = "%s"' % gridmode)

    def set_parameter(
        self,
        parameter="P1",
        param_engine="Maximum",
        source1="C1",
        source2=None,
        show_table=True,
    ):
        """Possible param_engine values listed in a table on page 1-151 of the automation manual.
        Some sample param_engine values are:
        Frequency / LevelAtX / Fall / Maximum / Mean / Median / Minimum / PeakToPeak"""
        self.vbs_write("app.Measure.ShowMeasure = %s" % show_table)
        self.vbs_write('app.Measure.%s.ParamEngine = "%s"' % (parameter, param_engine))
        if source1 is not None:
            self.vbs_write('app.Measure.%s.Source1 = "%s"' % (parameter, source1))
        if source2 is not None:
            self.vbs_write('app.Measure.%s.Source2 = "%s"' % (parameter, source2))
        self.vbs_write("app.Measure.%s.View = True" % parameter)

    def set_math(
        self, math_channel="F1", operator="AbsoluteValue", source1="C1", source2=None
    ):
        """Possible operator values listed in a table on page 1-151 of the automation manual.
        Sample values include: Average / Trend / Histogram / FFT / Integral / etc"""
        self.vbs_write('app.Math.%s.Operator1 = "%s"' % (math_channel, operator))
        if source1 is not None:
            self.vbs_write('app.Math.%s.Source1 = "%s"' % (math_channel, source1))
        if source2 is not None:
            self.vbs_write('app.Math.%s.Source1 = "%s"' % (math_channel, source2))

    def set_deskew(self, math_channel="F1", skew=10e-9):
        self.vbs_write(f"app.Math.{math_channel}.Operator1Setup.WaveDeskew = {skew}")

    def set_aux_trigger_out(self, status="on"):
        if status == "on":
            self.vbs_write('app.Acquisition.AuxOutput.AuxMode = "TriggerOut"')
        if status == "off":
            self.vbs_write('app.Acquisition.AuxOutput.AuxMode = "Off"')

    def set_measurement_clock_level(
        self, measurement="P1", clock="1", levelIs="Absolute", level=0.1
    ):
        self.vbs_write(
            "app.Measure.%s.Operator.Clock%sLevelIs = %s"
            % (measurement, clock, levelIs)
        )
        if levelIs == "Absolute":
            self.vbs_write(
                "app.Measure.%s.Operator.Clock%sAbsLevel = %s"
                % (measurement, clock, level)
            )
        if levelIs == "Percent":
            self.vbs_write(
                "app.Measure.%s.Operator.Clock%sPctLevel = %s"
                % (measurement, clock, level)
            )

    def set_measurement_gate(self, measurement="P1", gate_start=0, gate_stop=10):
        self.vbs_write(f"app.Measure.{measurement}.GateStart = {gate_start}")
        self.vbs_write(f"app.Measure.{measurement}.GateStop = {gate_stop}")

    def get_parameter_value(self, parameter="P1"):
        return float(self.vbs_ask("app.Measure.%s.Out.Result.Value" % parameter))

    def get_parameter_mean(self, parameter="P1"):
        return float(
            self.vbs_ask('app.Measure.Measure("%s").mean.Result.Value' % parameter)
        )

    def get_parameter_std(self, parameter="P1"):
        return float(
            self.vbs_ask('app.Measure.Measure("%s").sdev.Result.Value' % parameter)
        )

    def get_trigger_mode(self):
        return self.vbs_ask("app.Acquisition.TriggerMode")

    def get_wf_data(self, channel="C1"):  # e.g. channel = C1 or F3 etc
        ADDR_VGAIN = 156
        ADDR_VOFFSET = 160
        ADDR_HINTERVAL = 176
        ADDR_HOFFSET = 180
        self.write(channel + ":WAVEFORM? DAT1")  # Contains waveform data
        databytes = self.read_raw()
        databytes = databytes[16:]  # Strips leading header data
        if len(databytes) == 0:
            return np.array([]), np.array([])
        if len(databytes) % 2 == 1:  # Sometimes accidentally returns an extra byte
            databytes = databytes[:-1]
        data = np.frombuffer(databytes, np.int16)
        # DESC contains info about vertical/horiz scaling, etc,
        # see P280, Appendix II Remote Control Manual for byte addresses
        self.write(channel + ":WAVEFORM? DESC")  # Contains waveform data
        desc = self.read_raw()
        desc = desc[16:]
        vgain = float(np.frombuffer(desc[ADDR_VGAIN : ADDR_VGAIN + 4], np.float32))
        voffset = float(
            np.frombuffer(desc[ADDR_VOFFSET : ADDR_VOFFSET + 4], np.float32)
        )
        hinterval = float(
            np.frombuffer(desc[ADDR_HINTERVAL : ADDR_HINTERVAL + 4], np.float32)
        )
        hoffset = float(np.frombuffer(desc[ADDR_HOFFSET : ADDR_HOFFSET + 8], np.double))
        num_samples = len(data)
        x = np.array(range(num_samples)) * hinterval + hoffset
        # yscale = float(2**16)/(vf_stop - vf_start)  # Scale the data, it's output as ints from -2^16 to 2^16
        y = data * vgain - voffset
        return x, y

    def get_event_time(self, channel="C1"):
        vt_cy_time = self.vbs_ask(
            "app.Acquisition.%s.Out.Result.FirstEventTime" % channel
        )
        timestamp = self.decode_time(vt_cy_time)
        return vt_cy_time, timestamp

    def decode_time(self, vt_cy_time):
        """
        Parameters
        ----------
        vt_cy_time : VT_CY time from scope specs

        Returns
        -------
        datetime

        """
        epoch = datetime.datetime(2000, 1, 1)
        nanoseconds_since_epoch = int(float(vt_cy_time[:-2]) * 10000)
        delta = datetime.timedelta(microseconds=(nanoseconds_since_epoch // 1000))
        timestamp = epoch + delta
        formatted_timestamp = timestamp.strftime("%Y-%m-%d %H:%M:%S.%f")
        # print(formatted_timestamp)
        return timestamp

    def get_single_trace(self, channel="C1", save_data=True):
        """Sets scope to "single" trigger mode to acquire one trace, then waits until the trigger has happened
        (indicated by the trigger mode changing to "Stopped").  Returns blank lists if no trigger occurs within 1 second"""
        n = 0
        x = np.array([])
        y = np.array([])
        self.set_trigger_mode(trigger_mode="Single")
        while self.get_trigger_mode() == "Single" or n > 1e10:
            time.sleep(1e-4)
            n = n + 1
        if save_data:
            x, y = self.get_wf_data(channel=channel)
            return x, y
        else:
            pass

    # def get_math_data(self,channel='C1'):  # e.g. channel = C1 or F3 etc
    # return self.vbs_ask('app.Math.%s.Out.Result.Sweeps' % channel)

    def get_num_sweeps(self, channel="F1"):  # For use with histograms, trends, etc
        return int(self.vbs_ask("app.Math.%s.Out.Result.Sweeps" % channel))

    def set_num_sweeps(
        self,
        math_channel="F1",
        num_values=10e3,
    ):
        self.vbs_write(
            "app.Math.%s.Operator1Setup.Values = %s" % (math_channel, num_values)
        )

    def get_num_data_points(self, channel="P1"):
        return int(self.vbs_ask("app.Measure.%s.num.Result.Value" % channel))

    # def save_screenshot(self, filename = 'Hellokitty', filepath = 'C:\\LecroyScreenshots\\', grid_area_only = True):
    #     self.vbs_write('app.Hardcopy.Destination = "File"')
    #     if grid_area_only is True:
    #         self.vbs_write('app.Hardcopy.HardcopyArea = "GridAreaOnly"')
    #     else:
    #         self.vbs_write('app.Hardcopy.HardcopyArea = "DSOWindow"')
    #     self.vbs_write('app.Hardcopy.PreferredFilename = "%s"' % filename)
    #     self.vbs_write('app.Hardcopy.ImageFileFormat = "PNG"')
    #     self.vbs_write('app.Hardcopy.Directory = "%s"' % filepath)
    #     self.vbs_write('app.Hardcopy.Print')

    def set_math_vertical_scale(
        self, math_channel="F1", vertical_scale=10e-9, vertical_offset=0
    ):
        self.vbs_write(
            "app.Math.%s.Operator1Setup.VerScale = %s" % (math_channel, vertical_scale)
        )
        self.vbs_write(
            "app.Math.%s.Operator1Setup.Center = %s" % (math_channel, vertical_offset)
        )

    def setup_math_trend(self, math_channel="F1", source="P1", num_values=10e3):
        self.set_math(math_channel=math_channel, operator="Trend", source1=source)
        self.vbs_write(
            "app.Math.%s.Operator1Setup.Values = %s" % (math_channel, num_values)
        )
        self.view_channel(channel=math_channel, view=True)

    def setup_math_wf_average(self, math_channel="F1", source="C1", num_sweeps=100):
        self.set_math(math_channel=math_channel, operator="Average", source1=source)
        self.vbs_write(
            "app.Math.%s.Operator1Setup.Sweeps = %s" % (math_channel, num_sweeps)
        )
        self.view_channel(channel=math_channel, view=True)

    def setup_math_histogram(
        self,
        math_channel="F1",
        source="P1",
        num_values=10e3,
        num_bins=100,
        center=0,
        width_per_div=1,
        auto_scale=True,
    ):
        self.set_math(math_channel=math_channel, operator="Histogram", source1=source)
        self.vbs_write(
            "app.Math.%s.Operator1Setup.Values = %s" % (math_channel, num_values)
        )
        self.vbs_write(
            "app.Math.%s.Operator1Setup.AutoFindScale = %s" % (math_channel, auto_scale)
        )
        self.vbs_write(
            "app.Math.%s.Operator1Setup.Bins = %s" % (math_channel, num_bins)
        )
        self.vbs_write(
            "app.Math.%s.Operator1Setup.Center = %s" % (math_channel, center)
        )
        width_per_div = self.round_up_lockstep(width_per_div)
        self.vbs_write(
            "app.Math.%s.Operator1Setup.HorScale = %s" % (math_channel, width_per_div)
        )
        self.view_channel(channel=math_channel, view=True)

    def set_math_trend_values(self, math_channel="F1", num_values=1e3):
        self.vbs_write(
            "app.Math.%s.Operator1Setup.Values = %s" % (math_channel, num_values)
        )

    def math_histogram_clear_sweeps(self, math_channel="F1"):
        self.vbs_write("app.Math.%s.ClearSweeps" % math_channel)

    def histogram_find_center_width(self, math_channel="F1"):
        self.vbs_write("app.Math.%s.Operator1Setup.FindScale" % math_channel)

    def histogram_set_horiz_scale(self, math_channel="F1", horiz_scale=2e-6):
        self.vbs_write(
            "app.Math.%s.Operator1Setup.HorScale = %s" % (math_channel, horiz_scale)
        )

    def collect_sweeps(self, channel="F1", num_sweeps=1000):
        self.clear_sweeps()
        time.sleep(0.1)
        while self.get_num_sweeps(channel=channel) < num_sweeps + 1:
            time.sleep(0.1)
        x, ic_values = self.get_wf_data(channel=channel)
        while len(ic_values) < num_sweeps:
            time.sleep(0.05)
            x, ic_values = self.get_wf_data(channel=channel)
        return ic_values[
            :num_sweeps
        ]  # will occasionally return 1-2 more than num_sweeps

    def save_screenshot(self, file_name=None, white_background=True):
        if file_name is None:
            time_str = datetime.datetime.now().strftime("%Y-%m-%d %H-%M-%S")
            file_name = time_str + " lecroy screenshot.png"

        # lecroy.query('HCSU?') # Asks what the current hard copy setup is (useful for reference)
        if white_background:
            self.write("HCSU BCKG,WHITE")  # Sets background to white
        if white_background is False:
            self.write("HCSU BCKG,BLACK")  # Sets background to black
        self.write("HCSU DEV,PNG")
        self.write("HCSU FORMAT,LANDSCAPE")
        self.write("HCSU DEST,REMOTE")
        self.write("HCSU AREA,DSOWINDOW")
        self.write("SCREEN_DUMP")  # Takes the screenshot
        newFileBytes = self.read_raw()

        with open(file_name, "wb") as newFile:
            newFileByteArray = bytearray(newFileBytes)
            newFile.write(newFileByteArray)

        return file_name

    def get_single_trace_sequence(self, channel="C1", NumSegments=1000):
        """Sets scope to "single" trigger mode to acquire one trace, then waits until the trigger has happened
        (indicated by the trigger mode changing to "Stopped")."""

        self.set_sequence_mode()
        self.set_segments(NumSegments)
        self.clear_sweeps()
        self.set_trigger_mode(trigger_mode="Single")
        if self.get_trigger_mode() == "Single\n":
            while self.get_trigger_mode() == "Single\n":
                sleep(1e-4)
        x, y = self.get_wf_data(channel=channel)
        interval = abs(x[0] - x[1])
        xlist = []
        ylist = []
        totdp = np.int64(np.size(x) / NumSegments)
        for j in range(NumSegments):
            xlist.append(x[0 + j * totdp : totdp + j * totdp] - totdp * interval * j)
            ylist.append(y[0 + j * totdp : totdp + j * totdp])

        data_dict = {channel + "x": xlist, channel + "y": ylist}
        return data_dict

    def get_multiple_trace_sequence(self, channels=["C1", "C2"], NumSegments=1000):
        """
        If display mode (overlay, waterfall, adjacent) is not active turn off persistent trace

        """
        full_dict = {}
        self.set_sequence_mode()
        self.set_segments(NumSegments)
        # self.clear_sweeps()
        self.set_trigger_mode(trigger_mode="Single")
        if self.get_trigger_mode() == "Single\n":
            while self.get_trigger_mode() == "Single\n":
                sleep(1e-4)
        for c in channels:
            print(c)
            x, y = self.get_wf_data(channel=c)
            interval = abs(x[0] - x[1])
            xlist = []
            ylist = []
            totdp = np.int64(np.size(x) / NumSegments)

            for j in range(NumSegments):
                xlist.append(
                    x[0 + j * totdp : totdp + j * totdp] - totdp * interval * j
                )
                ylist.append(y[0 + j * totdp : totdp + j * totdp])

            data_dict = {c + "x": xlist, c + "y": ylist}
            full_dict.update(data_dict)

        return full_dict

    def my_get_single_trace_sequence(self, channel="C1", NumSegments=1000):
        """Sets scope to "single" trigger mode to acquire one trace, then waits until the trigger has happened
        (indicated by the trigger mode changing to "Stopped")."""
        self.set_sequence_mode()
        self.set_segments(NumSegments)
        self.set_trigger_mode(trigger_mode="Single")
        if self.get_trigger_mode() == "Single\n":
            while self.get_trigger_mode() == "Single\n":
                # SRS.set_output(False)
                sleep(1e-4)
                # SRS.set_output(True)
        x, y = self.get_wf_data(channel=channel)
        interval = abs(x[0] - x[1])
        xlist = []
        ylist = []
        totdp = np.int(np.size(x) / NumSegments)
        for j in range(NumSegments):
            xlist.append(x[0 + j * totdp : totdp + j * totdp] - totdp * interval * j)
            ylist.append(y[0 + j * totdp : totdp + j * totdp])

        return xlist, ylist

    def save_traces_multiple_sequence(
        self,
        channels=["C1", "C2"],
        num_traces=20,
        NumSegments=1000,
        threshold=[0, 0],
        fpath="",
        fname="myfile",
    ):
        """save multiple traces multiple times, threshold is set for each channel to eliminate false counts (set to )
        0 if unused"""
        num_ch = len(channels)
        x1list = []
        y1list = []
        x2list = []
        y2list = []
        x3list = []
        y3list = []
        x4list = []
        y4list = []
        q = 0  # number of not useless points
        for i in range(num_traces):
            print("Trace %d of %d" % (i, num_traces))
            try:
                if num_ch > 0:
                    x, y = self.my_get_single_trace_sequence(channels[0], NumSegments)
                    if max(abs(y[0])) > threshold[0]:
                        x1list.append(x)
                        y1list.append(y)
                    else:
                        q = q + 1
                        print("Not useful")
                if num_ch > 1:
                    x, y = self.my_get_single_trace_sequence(channels[1], NumSegments)
                    if max(abs(y[0])) > threshold[1]:
                        x2list.append(x)
                        y2list.append(y)
                    else:
                        q = q + 1
                        print("Not useful")
                if num_ch > 2:
                    x, y = self.my_get_single_trace_sequence(channels[2], NumSegments)
                    if max(abs(y[0])) > threshold[2]:
                        x3list.append(x)
                        y3list.append(y)
                    else:
                        q = q + 1
                        print("Not useful")
                if num_ch > 3:
                    x, y = self.my_get_single_trace_sequence(channels[3], NumSegments)
                    if max(abs(y[0])) > threshold[3]:
                        x4list.append(x)
                        y4list.append(y)
                    else:
                        q = q + 1
                        print("Not useful")
                if i % 100 == 0:
                    (print(i),)
            except Exception:
                print("error")

        data_dict = {}

        if num_ch > 0:
            x1list = np.reshape(
                x1list, (len(x1list) * len(x1list[0]), len(x1list[0][0]))
            )
            y1list = np.reshape(
                y1list, (len(y1list) * len(y1list[0]), len(y1list[0][0]))
            )
            data_dict[channels[0] + "x"] = x1list
            data_dict[channels[0] + "y"] = y1list
        if num_ch > 1:
            x2list = np.reshape(
                x2list, (len(x2list) * len(x2list[0]), len(x2list[0][0]))
            )
            y2list = np.reshape(
                y2list, (len(y2list) * len(y2list[0]), len(y2list[0][0]))
            )
            data_dict[channels[1] + "x"] = x2list
            data_dict[channels[1] + "y"] = y2list
        if num_ch > 2:
            x3list = np.reshape(
                x3list, (len(x3list) * len(x3list[0]), len(x3list[0][0]))
            )
            y3list = np.reshape(
                y3list, (len(y3list) * len(y3list[0]), len(y3list[0][0]))
            )
            data_dict[channels[2] + "x"] = x3list
            data_dict[channels[2] + "y"] = y3list
        if num_ch > 3:
            x4list = np.reshape(
                x4list, (len(x4list) * len(x4list[0]), len(x4list[0][0]))
            )
            y4list = np.reshape(
                y4list, (len(y4list) * len(y4list[0]), len(y4list[0][0]))
            )
            data_dict[channels[3] + "x"] = x4list
            data_dict[channels[3] + "y"] = y4list

        """below saves the data and a screen shot. it has been removed such that 
        saving is consistent with qnnpy"""
        # if '.' in fname:
        #     if fname.split('.')[1] != 'mat':
        #         print('file extension error')
        # else:
        #     fname = fname+'.mat'
        # #add time to the file name
        # t = datetime.datetime.now()
        # tstr = t.strftime('%Y%m%d')+'_%02d%02d%02d_'% (t.hour, t.minute, t.second)
        # #save matlab file
        # ffull = os.path.join(fpath, tstr+fname)
        # scipy.io.savemat(ffull, mdict=data_dict)
        # #save screen shot
        # scname = os.path.join(fpath, tstr+fname.split('.')[0]+'.png')
        # self.save_screenshot(scname)

        return data_dict

    def recall_setup(self, setup=1):
        self.vbs_write(f"app.SaveRecall.Setup.RecallInternal{setup}")


# lecroy_ip = '18.62.10.141'
# lecroy = LeCroy620Zi("TCPIP::%s::INSTR" % lecroy_ip)
# x,y = lecroy.get_wf_data('F1')
# from matplotlib import pyplot as plt
# plt.plot(x,y)
# plt.show()



"""
Laser control scripts for Santec tunable laser
Scripts from the Loncar group at Harvard.
Consider deleting if not needed.
"""
# import os
# import time
# from datetime import datetime

import nidaqmx
import numpy as np

# from nidaqmx.constants import TaskMode
# import yaml

# import sys
# from ..instruments.santec_tsl import SantecTSL
# import ..models.modeltools as modeltools


def single_scan(
    laser,
    daq_ai,
    daq_trig,
    wl_start=1550,
    wl_stop=1560,
    samples=100000,
    attenuation=20,
    scan_speed=20.0,
    backwards=False,
    timeout=5.0,
):
    # private single scan function, returns scandata
    wavstart = wl_start
    wavstop = wl_stop
    if not (
        (laser.wav_min <= wavstart <= laser.wav_max)
        and (laser.wav_min <= wavstop <= laser.wav_max)
    ):
        wl = None
        pd = None
        config = None
        print(
            "input wavelength out of range. wavelength min: {}nm, wavelength max: {} nm".format(
                laser.wav_min, laser.wav_max
            )
        )
        return wl, pd, config

    scandata = {}
    if backwards:
        wavstart = wl_stop
        wavstop = wl_start
    laser.setAttenuation(attenuation)
    t_acq = np.abs(wavstart - wavstop) / scan_speed  # expected acquisition time
    print("tacq: {:.3}".format(t_acq))
    sample_rate = float(samples / t_acq)
    wavs = np.linspace(wavstart, wavstop, samples)
    scandata["wav"] = wavs

    # configure daq task
    sweep_task = nidaqmx.Task()
    sweep_task.ai_channels.add_ai_voltage_chan(daq_ai)
    max_sample_rate = sweep_task.timing.samp_clk_max_rate
    if sample_rate > max_sample_rate:
        sweep_task.close()
        msg = "The requested sample rate {} is faster than the max rate {}".format(
            sample_rate, max_sample_rate
        )
        msg += ". Adjust scan settings to lower sample rate."
        raise Exception(msg)
    sweep_task.timing.cfg_samp_clk_timing(float(sample_rate), samps_per_chan=samples)
    sweep_task.triggers.reference_trigger.cfg_dig_edge_ref_trig(
        trigger_source=daq_trig, pretrigger_samples=2
    )
    # run scan
    sweep_task.start()
    try:
        laser.setStartWavelength(wavstart)
        laser.setStopWavelength(wavstop)
        laser.setSweepSpeed(scan_speed)
        laser.setSweepMode(1)
        laser.setSweepCycles(numOfSweeps=1)
        laser.setTriggerOut(triggerOut=True)
        laser.startSweep()
        timeout = timeout
        voltages = sweep_task.read(samples, timeout=t_acq + timeout)
        scandata["wav"] = wavs
        scandata["V"] = voltages
    except nidaqmx.errors.DaqError:
        print("Daq error in LaserCtrl._single_scan")
        # scandata['V'] = np.zeros(len(wavs))
        scandata = scandata
    sweep_task.close()
    wl = scandata["wav"]
    pd = scandata["V"]
    config = {
        "samples": samples,
        "scan_speed": scan_speed,
        "attenuation": attenuation,
        "timeout": timeout,
    }
    return wl, pd, config


#
# class LaserCtrl:
#     """LaserCtrl uses a laser and photodiode to perform monitor and scan
#     transmission data.
#     """
#     def __init__(self, laser, daq_address):
#         self.laser = laser  # laser object
#         #self.config = {}
#         #self.stop = False
#         #self.loop_scanning = False
#         #self.most_recent_scan = 'One way'
#         #self.sweep_task = None
#         #arr0 = np.array([0.0, ])
#         # initialize empty datasets
#         #self.scandata = {'wav': arr0, 'V': arr0, }
#         #self.scandata2way = {'wavstartstop': arr0, 'Vstartstop': arr0,
#                              'wavstopstart': arr0, 'Vstopstart': arr0}
#         #self.monitordata = [0, ]
#         #self.monitor_nsteps = 10  # number of points to scroll
#         #path_here = os.path.dirname(__file__)
#         #self._default_config_path = os.path.join(path_here,
#         #                         'config_defaults/laserctrl_default.yml')
#         #self.loaded_config_path = None

#    def load_config(self, fpath=None):
#        # load YAML file from fpath or default if fpath=None
#        self.config = modeltools.load_config(fpath,
#                                    defaultpath=self._default_config_path)
#        if fpath == None:
#            self.loaded_config_path = self._default_config_path
#        else:
#            self.loaded_config_path = fpath

#    def overwrite_config(self):
#        # overwrite YAML file used to load_config with current config
#        modeltools.save_config(self.loaded_config_path, self.config)

# def load_laser(self, laser=None):
#     # load laser, if None given, autoload from config
#     if laser is None:
#         kind = self.config['Laser']['kind']
#         rsc_name = self.config['Laser']['rsc_name']
#         if kind == 'SantecTSLDummy':
#             self.laser = SantecTSLDummy(rsc_name)
#         elif kind == 'SantecTSL':
#             self.laser = SantecTSL(rsc_name)
#         else:
#             raise Exception('Laser kind not recoginized in config')
#         self.laser.initialize()
#     else:
#         self.laser = laser
#         self.config['Laser']['kind'] = str(self.laser)
#         self.config['Laser']['rsc_name'] = self.laser.rsc_name

#    def monitor(self):
#        """Monitor photoreceiver voltage in time.
#        Config Args:
#            Monitor.tstep - timestep in sec, float
#            Monitor.scroll_time - length of stored data in sec, float
#        Updates:
#            monitordata - voltage measurements, 1D list
#        Notes:
#            + This function runs forever. You'll want to run it in a thread
#            + Can be stopped during acquisition by self.stop = True
#        """
#        print('Starting monitoring')
#        scroll_time = self.config['Monitor']['scroll_time']
#        tstep = self.config['Monitor']['tstep']
#        self.monitor_nsteps = int(scroll_time/tstep)
#        # setup analog read daq task
#        daqtask = nidaqmx.Task()
#        daqtask.ai_channels.add_ai_voltage_chan(self.config['DAQ']['ai'])
#        # init data to empty list
#        self.monitordata = []
#        self.stop = False
#        while not self.stop:
#            time.sleep(tstep)
#            datum = daqtask.read()  # next measurement
#            if len(self.monitordata) < self.monitor_nsteps:
#                self.monitordata = self.monitordata + [datum]
#            else:
#                self.monitordata[:-1] = self.monitordata[1:]
#                self.monitordata[-1] = datum
#        self.stop = False  # reset stop signal
#        daqtask.close()

# def wav_valid(self, wav):
#     return self.laser.wav_min <= wav <= self.laser.wav_max


# def stop_scan(self):
#     # abort scan mid-way
#     self.laser.stopSweep()
#     self.sweep_task.control(TaskMode.TASK_ABORT)
#     print('Scan aborted')

# def scan(self, backwards=False):
#     """Perform single continuous wavelength scan
#     Args:
#         backwards=False: if True, scan from wavstop to wavstart
#     Config Args:
#         Scan.wavestart - start wavelength in nm, float
#         Scan.wavstop - stop wavelength in nm, float
#         Scan.samples - number of samples to collect, int
#         Scan.speed - scan speed in nm/s
#     Updates:
#         scandata['wav'] - measured wavelengths in nm, array of float
#         scandata['V'] - measured voltages in V, array of float
#     Notes:
#         Wavelengths are estimated by assuming linear scan in time
#     """
#     print('Doing a one-way scan!')
#     self.scandata = self._single_scan(backwards=backwards)
#     self.most_recent_scan = 'One way'
#     if not self.loop_scanning:
#         time.sleep(0.2)
#         self.laser.setWavelength(self.config['LaserSettings']['wavelength'])

# def scan_loop(self, two_way=False):
#     """Perform scan in a loop
#     Config Args:
#         same as LaserCtrl.scan
#         Scan.loop_delay - wait time between scans
#     Args:
#         two_way=False - do two-way scan using LaserCtr.scan_twoway
#     Updates:
#         scandata - see LaserCtrl.scan and LaserCtrl.scan_twoway
#     Notes:
#         + This function runs forever. You'll want to run it in a thread.
#         + Can be stopped during acquisition by self.stop=True
#     """
#     print('Starting scan loop with two_way={}'.format(two_way))
#     self.stop = False
#     self.loop_scanning = True
#     while not self.stop:
#         if two_way:
#             self.scan_twoway()
#         else:
#             self.scan()
#         time.sleep(self.config['Scan']['loop_delay'])
#     time.sleep(0.2)
#     self.laser.setWavelength(self.config['LaserSettings']['wavelength'])
#     self.stop = False
#     self.loop_scanning = False

# def clear_data(self, kind='One way'):
#     '''Reinitilize dataset of a particular type
#     Args:
#         type - str {One way, Two way, Monitor}
#     '''
#     arr0 = np.array([0.0, ])
#     if kind == 'One way':
#         self.scandata = {'wav': arr0, 'V': arr0, }
#     elif kind == 'Two way':
#         self.scandata2way = {'wavstartstop': arr0, 'Vstartstop': arr0,
#                              'wavstopstart': arr0, 'Vstopstart': arr0}
#     elif kind == 'Monitor':
#         self.monitordata = [0,]

# def save_scandata(self,  save_config=True, overwrite_protect=True, saveplot=True):
#     # save data, using dir and fname args in config['Save']
#     savedir = self.config['Save']['dir']
#     fname = self.config['Save']['fname']
#     fpath = modeltools.overwrite_protection(savedir, fname,
#                     extension='.dat', overwrite_protect=overwrite_protect)
#     header = 'Data saved by LaserCtrl\n'
#     if self.most_recent_scan == 'One way':
#         modeltools.save_data_dict(data_dict=self.scandata, fpath=fpath,
#                   keys=['wav', 'V'],
#                   units=['nm', 'V'],
#                   col_labels=['wavelength', 'PD voltage'],
#                   header=header,
#                   config_dict=(self.config if save_config else None))
#     elif self.most_recent_scan == 'Two way':
#         modeltools.save_data_dict(data_dict=self.scandata2way, fpath=fpath,
#                   keys=['wavstartstop', 'Vstartstop', 'wavstopstart', 'Vstopstart'],
#                   units=['nm', 'V', 'nm', 'V'],
#                   col_labels=['wavelength start2stop', 'PD voltage start2stop',
#                               'wavelength stop2start', 'PD voltage stop2start'],
#                   header=header,
#                   config_dict=(self.config if save_config else None))
#
#     print('Data saved to: {}'.format(fpath))
#     if saveplot:
#         if self.most_recent_scan == 'One way':
#             xs = self.scandata['wav']
#             ys = self.scandata['V']
#             legend = None
#         elif self.most_recent_scan == 'Two way':
#             xs = [self.scandata2way['wavstartstop'], self.scandata2way['wavstopstart']]
#             ys = [self.scandata2way['Vstartstop'], self.scandata2way['Vstopstart']]
#             legend = ['start>stop', 'stop>start']
#         xlabel = 'Wavelength (nm)'
#         ylabel = 'PD Voltage (V)'
#         modeltools.save_data_fig(xs, ys, fpath, xlabel, ylabel, legend)
#

# def save_config(self, overwrite_protect=True):
#     # save config,using dir and fname args in config['Save']
#     fname = self.config['Save']['fname']
#     savedir = self.config['Save']['dir']
#     fpath = modeltools.overwrite_protection(
#                 savedir, fname, extension='.yml', overwrite_protect=overwrite_protect)
#     modeltools.save_config(fpath, self.config)



from time import sleep

import numpy as np
import pyvisa


class KeysightN5224a(object):
    """Python class for KeysightN5224a network analyzer, written by Di Zhu/Owen Medeiros.
    Connect the GPIB cable to GPIB1 (talker and listener)

    http://na.support.keysight.com/pna/help/latest/Programming/GP-IB_Command_Finder/SCPI_Command_Tree.htm
    https://helpfiles.keysight.com/csg/N52xxB/Programming/New_Programming_Commands.htm


    """

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands
        # set the data format; this seems to be the only format recognizable
        self.pyvisa.write("FORM ASCii,0")

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def reset(
        self,
        measurement="S21",
        if_bandwidth=1e3,
        start_freq=100e6,
        stop_freq=15e9,
        power=-30,
    ):
        # these two lines delete the calibration.
        # self.write('SYST:PRES')
        # self.write("CALC:PAR:DEL:ALL")

        # self.write("CALC:PAR:DEL") #this command deletes the measurement before setting the measurement below
        # self.set_measurement(measurement)
        self.set_start(start_freq)
        self.set_stop(stop_freq)
        self.set_if_bw(if_bandwidth)
        self.set_sweep_mode("CONT")
        self.set_power(power)

    def set_measurement(self, measurement="S21"):
        current_trace = self.get_trace_catalog()
        if current_trace == "NO CATALOG":
            self.write('CALC:PAR:DEF "{}", {}'.format(measurement, measurement))
            self.write('CALC:PAR:SEL "{}"'.format(measurement))
        self.write('CALC:PAR:MOD:EXT "{}"'.format(measurement))
        trace_name = self.get_trace_catalog()
        trace_name = trace_name.split(",")[0]
        self.write("DISPlay:WINDow1:TRACe1:FEED '{}'".format(trace_name))

    def set_basic(
        self,
        cnum,
        start_freq=100e6,
        stop_freq=15e9,
        power=-30,
        if_bandwidth=1e3,
        points=1001,
    ):
        self.write("SENS{}:FREQ:STAR {}".format(cnum, start_freq))
        self.write("SENS{}:FREQ:STOP {}".format(cnum, stop_freq))
        self.write("SOUR{}:POW {:.2f}".format(cnum, power))
        self.write("SENS{}:BAND {:.2f}".format(cnum, if_bandwidth))
        self.write("SENS{}:SWE:POIN {}".format(cnum, points))

    def set_Sparam(
        self, start_freq=100e6, stop_freq=15e9, power=-30, if_bandwidth=1e3, points=1001
    ):
        self.write("CALC:PAR:DEL:ALL")
        self.write("CALCulate1:PARameter:DEFine:EXT 'Meas1','S11'")
        self.write("CALCulate1:PARameter:DEFine:EXT 'Meas2','S21'")
        self.write("CALCulate1:PARameter:DEFine:EXT 'Meas3','S12'")
        self.write("CALCulate1:PARameter:DEFine:EXT 'Meas4','S22'")

        self.write("DISPlay:WINDow1:STATE ON")
        self.write("DISPlay:WINDow2:STATE ON")
        self.write("DISPlay:WINDow3:STATE ON")
        self.write("DISPlay:WINDow4:STATE ON")

        self.write("DISPlay:WINDow1:TRACe1:FEED 'Meas1'")
        self.write("DISPlay:WINDow2:TRACe1:FEED 'Meas2'")
        self.write("DISPlay:WINDow3:TRACe1:FEED 'Meas3'")
        self.write("DISPlay:WINDow4:TRACe1:FEED 'Meas4'")

        self.set_basic(1, start_freq, stop_freq, power, if_bandwidth, points)
        # self.set_basic(2, start_freq, stop_freq, power, if_bandwidth, points)
        # self.set_basic(3, start_freq, stop_freq, power, if_bandwidth, points)
        # self.set_basic(4, start_freq, stop_freq, power, if_bandwidth, points)

        self.set_sweep_mode("SING")
        # self.write("INITiate:IMMediate")
        while self.get_sweep_mode() != "HOLD":
            sleep(0.1)
        self.set_sweep_mode("CONT")
        self.set_scale_auto_all()

    def set_Sparam2(
        self, start_freq=100e6, stop_freq=15e9, power=-30, if_bandwidth=1e3, points=1001
    ):
        self.write("CALC:PAR:DEL:ALL")
        self.write("CALCulate1:PARameter:DEFine:EXT 'Meas1','S21'")

        self.write("DISPlay:WINDow1:STATE ON")

        self.write("DISPlay:WINDow1:TRACe1:FEED 'Meas1'")

        self.set_basic(1, start_freq, stop_freq, power, if_bandwidth, points)
        self.set_basic(2, start_freq, stop_freq, power, if_bandwidth, points)
        self.set_basic(3, start_freq, stop_freq, power, if_bandwidth, points)
        self.set_basic(4, start_freq, stop_freq, power, if_bandwidth, points)

        self.set_sweep_mode("SING")
        # self.write("INITiate:IMMediate")
        while self.get_sweep_mode() != "HOLD":
            sleep(0.1)
        self.set_sweep_mode("CONT")
        self.set_scale_auto_all()

    def single_sweep(self, channel=1):
        """run a single sweep and get f in Hz, S in real + i*imag form"""
        # self.write("INITiate:CONTinuous OFF")
        self.set_sweep_mode("SING", channel)
        # self.write("INITiate:IMMediate")
        while self.get_sweep_mode() != "HOLD":
            sleep(0.1)
        print(1)
        f = self.get_frequency(channel)
        S = self.read_channel(channel)
        return f, S

    def read_channel(self, channel=1):
        val = self.query("CALC{}:DATA? SDATA".format(channel))
        val = [x.strip() for x in val.split(",")]
        val = [float(x) for x in val]
        real_part = np.array(val[0::2])
        imag_part = np.array(val[1::2])
        S = real_part + 1j * imag_part
        return S

    # strip \n and return a float. OM - i think float() handles \n
    def _pna_getter(self, s):
        return float(s.strip())

    def get_frequency(self, channel=1):
        print(11)
        freq = self.query("CALC{}:X?".format(channel))
        print(12)
        freq = [x.strip() for x in freq.split(",")]
        print(13)
        freq = [float(x) for x in freq]
        print(14)
        return np.array(freq)

    def select_measurement(self, measurement_name="CH1_S11_1"):
        self.write("CALCulate:PARameter:SELect '" + measurement_name + "'")

    # IF bandwidth
    def get_if_bw(self):
        cmd = "SENS:BAND?"
        return self._pna_getter(self.query(cmd))

    def set_if_bw(self, bandwidth=1000):
        self.write("SENS:BAND {:.2f}".format(bandwidth))

    # power
    def get_power(self):
        cmd = "SOUR:POW?"
        return self._pna_getter(self.query(cmd))

    def set_power(self, power=-10):
        """set power in dbm"""
        self.write("SOUR:POW {:.2f}".format(power))

    # frequency range
    def set_start(self, frequency=100e6):
        cmd = "SENS:FREQ:STAR {}".format(frequency)
        self.write(cmd)

    def set_stop(self, frequency=20e9):
        cmd = "SENS:FREQ:STOP {}".format(frequency)
        self.write(cmd)

    def set_center(self, frequency=5e9):
        cmd = "SENS:FREQ:CENT {}".format(frequency)
        self.write(cmd)

    def set_span(self, frequency=5e9):
        cmd = "SENS:FREQ:SPAN {}".format(frequency)
        self.write(cmd)

    def set_points(self, points=201):
        cmd = "SENS:SWE:POIN {}".format(points)
        self.write(cmd)

    def set_calibration(self, state=1):
        """Set measurement must be run first.
        Note: Before using this command you must select a measurement using
        CALC:PAR:SEL. You can select one measurement for each channel.
        """
        cmd = "SENS:CORR {}".format(state)
        self.write(cmd)

    def set_scale_auto(self, window=1):
        cmd = "DISP:WIND{}:Y:AUTO".format(window)
        self.write(cmd)

    def set_scale_auto_all(self):
        """Scales all windows"""
        win = self.query("DISPlay:CATalog?")
        win = win[1:-2]
        for w in win.split(","):
            self.write("DISP:WIND{}:Y:AUTO".format(w))

    def get_start(self):
        cmd = "SENS:FREQ:STAR?;"
        return self._pna_getter(self.query(cmd))

    def get_stop(self):
        cmd = "SENS:FREQ:STOP?;"
        return self._pna_getter(self.query(cmd))

    def get_center(self):
        cmd = "SENS:FREQ:CENT?;"
        return self._pna_getter(self.query(cmd))

    def get_span(self):
        cmd = "SENS:FREQ:SPAN?;"
        return self._pna_getter(self.query(cmd))

    def get_sweep_time(self):
        cmd = "SENS:SWE:TIME?;"
        return self._pna_getter(self.query(cmd))

    # number of points in a sweep
    def get_points(self):
        cmd = "SENS:SWE:POIN?"
        return self._pna_getter(self.query(cmd))

    def get_trace_catalog(self):
        """
        Get the trace catalog, that is a list of trace and sweep types
        from the Pself.
        The format of the returned trace is:
            trace_name,trace_type,trace_name,trace_type...
        """
        return self.query("CALC:PAR:CAT:EXT?").strip().strip('"')

    def get_window_catalog(self):
        return self.query("DISPlay:CATalog?").strip().strip('"')

    # electrical delay
    # def get_electrical_delay():
    #    '''unit s'''
    #    get_cmd='CALC:CORR:EDEL:TIME?'
    #    return pna_getter(self.query(get_cmd))
    #
    # def set_electrical_delay(delay=0):
    #    set_cmd='CALC:CORR:EDEL:TIME {:.6e}'.format(delay)
    #    self.write(set_cmd)

    # sweep mode
    def set_sweep_mode(self, mode="CONT", cnum=1):
        """vals=Enum("HOLD", "CONT", "GRO", "SING"))"""
        set_cmd = "SENS{}:SWE:MODE {}".format(cnum, mode)
        self.write(set_cmd)

    def get_sweep_mode(self, cnum=1):
        """vals=Enum("HOLD", "CONT", "GRO", "SING"))"""
        get_cmd = "SENS{}:SWE:MODE?".format(cnum)
        return self.query(get_cmd).strip()

    def get_source_attenuation(self):
        get_cmd = "SOUR:POW:ATT?"
        return self._pna_getter(self.query(get_cmd))

    def get_freq_real_imag(self):
        # change to polar coordinates
        # here I assume the measurment is #1 (MNUM=1)
        # self.write('CALC:PAR:MNUM:SEL 1')
        # self.write('CALC:FORM POL')

        f_start = self.get_start()
        f_stop = self.get_stop()
        num_pts = self.get_points()

        print(
            "Sweeping from %0.0d MHz to %0.0d MHz, with %0.0d points"
            % (f_start / 1e6, f_stop / 1e6, num_pts)
        )

        # self.write('FORM:DATA REAL,64')
        # F = np.linspace(f_start, f_stop, num_pts, endpoint = True)
        # GET FREQUENCIES!

        # self.write('FORM:DATA ASC') #this is in _init_
        try:
            raw_f = self.query("CALC:X?")
            raw = self.query("CALC:DATA? FDATA")
        except Exception as e:
            print(
                'If SCPI command "unterminated", check that measurement has been selected'
            )

        f = str(raw_f).split(",")
        data = str(raw).split(",")

        real_arr = data[::2]  # Every other element starting with element 0
        imag_arr = data[1::2]

        freq = np.asarray(np.float_(f))
        real = np.asarray(np.float_(real_arr))
        imag = np.asarray(np.float_(imag_arr))

        return freq, real, imag



from time import sleep

import pyvisa


class Cryocon34(object):
    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def read_temp(self, channel="A"):
        self.write(":INPUT? " + channel + ":TEMP")  # In form of ":INPUT? A:TEMP", A-D
        try:
            temperature = float(self.read())
        except Exception:
            temperature = "no reading"

        return temperature

    def setup_heater(self, load=25, range="25W", source_channel="A"):
        self.write("LOOP 1:SETPT 10")
        self.write("LOOP 1:LOAD " + str(load))
        self.write("LOOP 1:RANGE " + range)
        self.write("LOOP 1:SOUR CH" + str(source_channel))
        self.write("LOOP 1:TYPE OFF")  # Default to off in case of trouble
        self.write("LOOP 1:PMAN 0")  # Default to zero manual power in case of trouble
        # Range can be 50W, 5.0W, 0.5W and 0.05W
        # or for 25ohm load 25W, 2.5W, 0.3W and 0.03W

    def set_pid(self, proportional_gain=0.5, integral_gain=5.0, derivative_gain=0.0):
        self.write("LOOP 1:PGAIN " + str(proportional_gain))
        self.write("LOOP 1:IGAIN " + str(integral_gain))
        self.write("LOOP 1:DGAIN " + str(derivative_gain))

    def set_setpoint(self, setpoint=5):
        self.write("LOOP 1:SETPT " + str(setpoint))
        print("- Changing heater setpoint to " + str(setpoint) + "K")
        sleep(1)

    def set_control_type_rampp(self, ramp_rate=1.0):
        """Ramps at a rate of ramp_rate kelvin per minute using PID control"""
        self.write("LOOP 1:TYPE RAMPP")
        ramp_rate_rounded = float(round(ramp_rate * 10)) / 10
        self.write("LOOP 1:RATE " + str(ramp_rate_rounded))  # Ramp rate in K/min

    def set_control_type_pid(self):
        """Goes to setpoint value using PID control"""
        self.write("LOOP 1:TYPE PID")

    def start_heater(self):
        self.write("SYSTEM:BEEP 1")
        self.write("CONTROL")

    def stop_heater(self):
        self.write("STOP")
        self.write("SYSTEM:BEEP 1")
        sleep(0.5)

    def heat_up(
        self,
        load=50,
        range="50W",
        source_channel="B",
        setpoint=293,
        proportional_gain=0.5,
        integral_gain=5.0,
        derivative_gain=0.0,
    ):
        self.set_pid(proportional_gain, integral_gain, derivative_gain)
        self.write("LOOP 1:SETPT " + str(setpoint))
        self.write("LOOP 1:LOAD " + str(load))
        self.write("LOOP 1:RANGE " + range)
        self.write("LOOP 1:SOUR CH" + str(source_channel))
        self.write("LOOP 1:TYPE Rampp")  # Default to off in case of trouble
        self.write("LOOP 1:PMAN 5")
        self.start_heater()

    # *CLS
    # *IDN?
    # SYSTEM:BEEP 1;*OPC?
    # LOOP 2:TYPE OFF;*OPC? # Turns off analog heater in loop 2
    # LOOP 1:TYPE PID;*OPC? # Sets PID mode
    # LOOP 1:TYPE RAMPP;*OPC? # Sets PID ramp mode
    # LOOP 1:SOUR CHD;*OPC? # Sets heater ("loop 1") channel e.g. CHB = Channel B
    # LOOP 1:SETPT 10;*OPC? # Sets desired temp set point
    # LOOP 1:RANGE 0.5W;*OPC?
    # LOOP 1:RANGE 5.0W;*OPC?
    # HEATER:AUTOTUNE:START # Must be in TYPE PID mode to run
    # LOOP 1:PGAIN 0.5;*OPC?
    # LOOP 1:IGAIN 5;*OPC?
    # LOOP 1:DGAIN 0;*OPC?
    # LOOP 1:SOURCE?
    # LOOP 1:SOUR CHA
    # LOOP 1:SETPT?
    # LOOP 1:TYPE?
    # LOOP 1:RANGE?
    # LOOP 1:LOAD?
    # LOOP 1:RAMP?
    # LOOP 1:RATE? # A rate of 0.02 means the heater will do 0.02K/min (if using Kelvin units)
    # CONTROL;*OPC?
    # STOP;*OPC?



from enum import Enum

import pyvisa


class MeasFunction(Enum):
    VOLT = 1
    VOLTAC = 2
    CURR = 3
    CURRAC = 4
    RES = 5  # 2pt res
    FRES = 6  # 4pt res
    FREQ = 7
    PER = 8
    TEMP = 9


class Keithley2700(object):
    """Python class for Keithley 2700 Data Acquisition System, written by Adam McCaughan"""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 50000000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def local_key(self):
        self.write("SYST:KEY 17")

    def reset(self):
        self.write("*RST")

    def read_voltage(self):
        self.write("FUNC 'VOLT'")
        read_str = self.query(":READ?")
        # Returns something like '+1.99919507E-01VDC,+6283.313SECS,+60584RDNG#'
        E_location = read_str.find("E")  # Finds location of E in read value
        voltage_str = read_str[0 : read_str.find("E") + 4]
        return float(voltage_str)  # Return just the first number (voltage)

    def read_resistance(self):
        self.write("FUNC 'RES'")
        read_str = self.query(":READ?")
        E_location = read_str.find("E")
        res_str = read_str[0 : read_str.find("E") + 4]
        return float(res_str)

    def set_Zin_10Mohm(self):
        self.write("VOLT:IDIV ON")

    def set_Zin_10Gohm(self):
        self.write("VOLT:IDIV OFF")

    def setup_filter(self, filter_type="MOV", window="0.1", count="100"):
        self.write(
            ":VOLT:AVER:TCON " + filter_type
        )  # Select filter type; # <name> = MOVing or REPeat. (Note 2)
        self.write(
            ":VOLT:AVER:WIND " + window
        )  # Set filter window in %; <NRf> = 0 to 10. 0.1
        self.write(
            ":VOLT:AVERage:COUN " + count
        )  # Specify filter count; <n> = 1 to 100. 10

    def set_DCV(self):
        self.write("VOLT:DC")

    def set_filter(self, state="OFF"):
        self.write(":VOLT:AVER:STAT " + state)  # Enable or disable the filter.

    def set_autorange(self, meas_fxn):
        if meas_fxn == MeasFunction.CURRAC:
            meas_str = "CURR:AC"
        elif meas_fxn == MeasFunction.VOLTAC:
            meas_str = "VOLT:AC"
        else:
            meas_str = str(meas_fxn)
        self.write(f"{meas_str}:RANG:AUTO ON")

    def resistance_configure4p(self):
        # Configure the instrument for 4-point resistance measurement
        self.pyvisa.write(
            "ROUTe:SCAN ((@101,102))"
        )  # Configure single channel for 2-point measurement
        self.pyvisa.write("SENS:FUNC 'FRES'")  # Set the function to measure resistance
        self.pyvisa.write("SENS:FRES:NPLC 10")  # Set integration time to 10 PLC
        self.pyvisa.write("SENS:FRES:RANG 10M")  # Enable auto-range

    def read_resistance4p(self):
        # Perform the 4-point measurement
        self.pyvisa.write("INIT")
        self.pyvisa.write("*WAI")  # Wait for the measurement to complete
        read_str = self.pyvisa.query("FETCh?")  # Query the resistance value
        E_location = read_str.find("E")
        res_str = read_str[0 : read_str.find("E") + 4]
        return float(res_str)



"""
instrument class for TEKTRONIX RSA5126A
Created on Feb 26, 2020
@author: dizhu
"""

import numpy as np
import pyvisa


class RSA5126A(object):
    """Python class for TEKTRONIX RSA5126A, written by Di Zhu."""

    def __init__(self, visa_name, timeout=15000):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = timeout  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def query_binary_values(self, string):
        return self.pyvisa.query_binary_values(string)

    def idn(self):
        return self.query("*IDN?")

    def reset(self):
        self.write("*RST")

    def get_freq_span(self):
        """return the frequency span in GHz"""
        string = "SENSE:SPECTRUM:FREQUENCY:SPAN?"
        return float(self.query(string).strip()) / 1e9

    def set_freq_span(self, span=1.0):
        """set freq span in GHz"""
        string = "SENSE:SPECTRUM:FREQUENCY:SPAN"
        self.write("{} {}".format(string, span * 1e9))

    def get_freq_center(self):
        """return the frequency span in GHz"""
        string = "SENSE:SPECTRUM:FREQUENCY:CENTER?"
        return float(self.query(string).strip()) / 1e9

    def set_freq_center(self, freq=1.0):
        """set freq span in GHz"""
        string = "SENSE:SPECTRUM:FREQUENCY:CENTER"
        self.write("{} {}".format(string, freq * 1e9))

    def get_spectrum(self, channel=None):
        if channel is None:
            string = "READ:SPEC:TRAC?"
        else:
            string = "READ:SPEC:TRAC{:d}?".format(channel)
        return self.query_binary_values(string)

    def single_scan(self):
        self.write("INITiate:CONTinuous OFF")
        self.write("INITiate:IMMediate")
        s = self.get_spectrum()
        f_start = float(self.query("SPECtrum:FREQuency:STARt?").strip()) / 1e9
        f_stop = float(self.query("SPECtrum:FREQuency:STOP?").strip()) / 1e9
        # f_step = float(self.query('SPECtrum:FREQuency:STEP?').strip())/1e9
        f = np.linspace(f_start, f_stop, len(s))
        return f, s



import time

import pyvisa


class AttocubeANC300(object):
    """Python class for Attocube ANC300 peizo controller, by Di Zhu 2016"""

    """functions follow the user manual """

    def __init__(self, visa_name, baud_rate=38400):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name, baud_rate=baud_rate)
        self.pyvisa.timeout = 5000  # set response time in milliseconds

    def reset(self):
        for i in range(1, 4):
            self.setf(i, 1000)
            self.setv(i, 10)
            self.setm(i, "stp")

    def read(self):
        results = ""
        readline = self.pyvisa.read()
        while readline[0:2] != "OK" and readline[0:4] != "ERROR":
            results = results + readline
            readline = self.pyvisa.read()
        return results

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        self.write(string)
        return self.read()

    def help(self):
        self.write("help")
        print(self.read())

    def ver(self):
        self.write("ver")
        print(self.read())

    def setm(self, axis=1, amode="stp"):
        # <amode> ext, stp, gnd cap
        self.write("setm " + str(axis) + " " + amode)
        self.read()  # clear out the buffer

    def stop(self, axis=1):
        # <amode> ext, stp, gnd cap
        self.write("stop " + str(axis))
        print(self.read())  # clear out the buffer

    def stepu(self, axis=1, steps=1):
        # step up
        self.write("stepu " + str(axis) + " " + str(steps))
        self.read()  # clear out the buffer

    def contu(self, axis=1):
        # continuous travel up
        self.write("stepu " + str(axis) + " c")
        self.read()  # clear out the buffer

    def stepd(self, axis=1, steps=1):
        # step down
        self.write("stepd " + str(axis) + " " + str(steps))
        self.read()  # clear out the buffer

    def contd(self, axis=1):
        # conitnous drive down
        self.write("stepd " + str(axis) + " c")
        self.read()  # clear out the buffer

    def setf(self, axis=1, freq=1000):
        # set frequency
        self.write("setf " + str(axis) + " " + str(freq))
        self.read()

    def setv(self, axis=1, voltage=10):
        # set voltage
        self.write("setv " + str(axis) + " " + str(voltage))
        self.read()

    def getf(self, axis=1):
        # read frequency
        return self.query("getf " + str(axis))

    def getv(self, axis=1):
        # read voltage
        return self.query("getv " + str(axis))

    def getc(self, axis=1):
        # read capacitance
        self.setm(axis, "cap")
        time.sleep(2)
        # time.sleep(1)
        c = self.query("getc " + str(axis))

        self.setm(axis, "stp")
        time.sleep(1)
        return float(c.split(" ")[4])

    def reset(self):
        for i in range(1, 4):
            self.setm(axis=i, amode="stp")
            self.setv(i, 10)
            self.setf(i, 1000)


# ac = AttocubeANC150(u'ASRL5::INSTR')
# ac.reset()

# ac.write('help')
# print ac.read()
# ac.query('help')



import datetime

import scipy.io
from matplotlib import pyplot as plt


def save_plot(
    x,
    y,
    filename,
    xlabel="",
    ylabel="",
    title="",
    legend=None,
    prepend_time=True,
    save_plot=True,
    show_plot=False,
):
    time_str = datetime.datetime.now().strftime("%Y-%m-%d %H-%M-%S")
    filename = time_str + " " + filename
    scipy.io.savemat(filename + ".mat", mdict={"x": x, "y": y})

    plt.plot(x, y, "-o")
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    if legend is not None:
        plt.legend(title)
    plt.savefig(filename)
    if show_plot is True:
        plt.show()
    plt.close()


def save_multi_plots(
    x1,
    y1,
    x2,
    y2,
    filename,
    xlabel="",
    ylabel="",
    title="",
    legend=None,
    prepend_time=True,
    save_plot=True,
    show_plot=False,
):
    time_str = datetime.datetime.now().strftime("%Y-%m-%d %H-%M-%S")
    filename = time_str + " " + filename
    scipy.io.savemat(filename + ".mat", mdict={"x1": x1, "y1": y1, "x2": x2, "y2": y2})

    plt.plot(x1, y1, "-bo")
    plt.plot(x2, y2, "-ro")
    plt.xlabel(xlabel)
    plt.ylabel(ylabel)
    plt.title(title)
    if legend is not None:
        plt.legend(legend)
    plt.savefig(filename)
    if show_plot is True:
        plt.show()
    plt.close()



"""
instrument class for EXFO T100S-HP tunable laser
Created on Feb 26, 2020
@author: dizhu
"""

import pyvisa


class EXFOT100S(object):
    """Python class for EXFO T100S-HP, written by Di Zhu."""

    def __init__(self, visa_name, timeout=15000):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = timeout  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def get_wl(self):
        return float(self.query("L?").strip().split("=")[1])

    def get_wl_min(self):
        return float(self.query("L? MIN").strip().split("=")[1])

    def get_wl_max(self):
        return float(self.query("L? MAX").strip().split("=")[1])

    def set_wl(self, wl=1550.0):
        wl_min = self.get_wl_min()
        wl_max = self.get_wl_max()
        if wl >= wl_min and wl <= wl_max:
            self.write("L {}".format(wl))
        else:
            print("wavelength out of range")

    def set_apc(self, apc=True):
        if apc:
            self.write("APCON")
        else:
            self.write("APCOFF")

    def reset(self):
        self.write("INIT")
        self.set_apc(apc=True)
        self.set_power_unit(unit="dBm")

    def set_output(self, output=True):
        if output:
            self.write("ENABLE")
        else:
            self.write("DISABLE")

    def set_power_unit(self, unit="dBm"):
        """set power unit: DBM or MW"""
        if unit == "dBm":
            self.write("DBM")
        if unit == "mW":
            self.write("MW")

    def get_power(self):
        power = self.query("P?").strip()
        if power == "DISABLED":
            print("disabled")
        else:
            return float(power.split("=")[1])

    def set_power(self, power=-20.0):
        self.write("P={:.2f}".format(power))



import time

import pyvisa


class FVA3100(object):
    """Python class for FVA3100 Optical Attenuator, written by Adam McCaughan."""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        self.wait_time = 5  # in second #this optical attenuator switches very slowly
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def set_attenuation_db(self, attenuation_db=-10):
        self.write(
            ("ATT %0.1f dB" % -attenuation_db)
        )  # this attenuator has a different convention; it takes negative value as attenuation
        time.sleep(self.wait_time)

    def set_beam_block(self, beam_block=True):
        # self.write(('D %0.0f' % beam_block))  #it appears that the beam blocker for this attenuator is broken. Instead, we set -80 dB attenuation
        if beam_block:
            self.write("ATT -80")
            time.sleep(self.wait_time)


# att = JDSHA9('GPIB0::10')



import pyvisa


class AnritsuMG9638A(object):
    """Python class for Antritsu M9638 tunable laser source, written by Adam McCaughan.  Adapted from
    Mihir's MATLAB code"""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def reset(self):
        self.write("*RST")

    def setup_basic(self):
        self.write("MCW")  # Set laser to CW mode
        self.write("COH ON")  # Set laser to coherent mode
        self.set_power_unit("mW")
        self.set_wavelength(1550)

    def set_power_unit(self, power_unit):
        """power_unit should be either dBm, mW, or uW"""
        self.write("POWU %s" % power_unit)

    def get_power_unit(self, power_unit):
        return self.query("POWU?")

    def get_wavelength(self):
        wavelength_str = self.query("OUTW?")
        return float(wavelength_str)

    def set_output(self, output=False):
        self.write("OUTP %d" % output)

    def get_output(self):
        return bool(self.query("OUTP?"))

    def set_power(self, power, power_unit="uW"):
        """power_unit should be either dBm, mW, or uW"""
        power_str = self.write("POW %0.3d%s" % (power, power_unit))

    def get_power(self):
        power_str = self.query("POW?")
        return float(power_str)

    def set_wavelength(self, lambda_nm):
        self.write("WCNT %0.3fNM" % lambda_nm)


# ls = AnritsuMG9638A('GPIB0::24')

# def setup_4W_source_I_read_V(self):
#     self.write('*RST')
#     self.write(':SOUR:FUNC CURR') # Set operation mode to: source current
#     self.write(':SOUR:CURR:LEVEL 0E-6') # Set current level to 0 uA
#     self.write(':SYST:RSEN 1') # Turn off "Remote Sensing" aka 4-wire measurement mode
#     self.write('SENS:FUNC \"VOLT\", \"CURR\"') # Have it output


# def setup_2W_source_I_read_V(self):
#     self.write('*RST')
#     self.write(':SOUR:FUNC CURR') # Set operation mode to: source current
#     self.write(':SOUR:CURR:LEVEL 0E-6') # Set current level to 0 uA
#     self.write(':SYST:RSEN 0') # Turn off "Remote Sensing" aka 4-wire measurement mode
#     self.write('SENS:FUNC \"VOLT\", \"CURR\"') # Have it output


# def setup_2W_source_V_read_I(self):
#     self.write('*RST')
#     self.write(':SOUR:FUNC VOLT') # Set operation mode to: source voltage
#     self.write(':SOUR:VOLT:LEVEL 0E-3') # Set voltage level to 0 mV
#     self.write(':SYST:RSEN 0') # Turn off "Remote Sensing" aka 4-wire measurement mode
#     self.write('SENS:FUNC \"VOLT\", \"CURR\"') # Have it output


# def set_output(self, output = False):
#     if output is True:  self.write(':OUTP ON')
#     if output is False: self.write(':OUTP OFF')


# def set_compliance_i(self, compliance_i = 10e-6):
#     self.write(':SENS:CURR:PROT %0.3e' % compliance_i)


# def set_compliance_v(self, compliance_v = 10e-6):
#     self.write(':SENS:VOLT:PROT %0.3e' % compliance_v)


# def set_current(self, current = 0e-6):
#     self.write(':SOUR:CURR:LEVEL %0.4e' % current)   # Set current level


# def set_voltage(self, voltage = 0e-6):
#     self.write(':SOUR:VOLT:LEVEL %0.4e' % voltage)   # Set current level


# def read_voltage_and_current(self):
#     read_str = self.query(':READ?')
#     # See page 18-51 of manual, returns: voltage, current, resistance, timestamp, status info
#     # Returns something like '5.275894E-05,-1.508318E-06,+9.910000E+37,+2.562604E+03,+3.994000E+04'
#     data = read_str.split(',')
#     voltage, current = float(data[0]), float(data[1])
#     return voltage, current


# def read_current(self, current = 0e-6):
#     voltage, current = self.read_voltage_and_current()
#     return current


# def read_voltage(self):
#     voltage, current = self.read_voltage_and_current()
#     return voltage



import os
import psutil
import time
import logging

# Setup logging
log_file = r'C:\Users\ICE\ice-logging\watchdog_logfile.txt'
logging.basicConfig(filename=log_file, level=logging.INFO, format='%(asctime)s %(message)s')

logging.info('Watchdog script started.')

# Define the flag file path using a raw string
flag_file = r'C:\Users\ICE\ice-logging\flagfile.txt'

# Check if the primary script is running
def is_primary_running():
    for proc in psutil.process_iter(['pid', 'name', 'cmdline']):
        if "Q:\qnnpy\qnnpy\functions\ice_logging_script.pyw" in proc.info['cmdline']:
            logging.info("Script is running")
            return True
        else:
            logging.info("script is not running")
            return False

# Watchdog loop
while True:
    if os.path.exists(flag_file):
        if not is_primary_running():
            logging.warning('Flag file exists but primary script is not running. Removing stale flag file.')
            os.remove(flag_file)
        else:
            logging.info("Primary script is running")
    time.sleep(10)  # Check every 10 seconds



import pyvisa


class SGS100A(object):
    """Python class for Agilent 53131a counter, written by Adam McCaughan
    Use like c = Agilent53131a('GPIB0::3')"""

    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def reset(self):
        self.write("*RST")
        self.write("*CLS")

    def basic_setup(self):
        self.reset()
        self.write(":SOURce:OPMode NORMal")
        self.write(":SOURce:FREQuency:CW 2 GHz")
        self.write(":SOURce:POWer -10dBm")
        self.write(":OUTPut:STATe OFF")

    def set_output_attenautor_mode(self, mode="AUTO"):
        """set the attenautor mode on the output
        AUTO | FIXed | APASsive"""
        self.write(":OUTPut:AMODe {}".format(mode))

    def set_output(self, state="OFF"):
        """activates/deactivates the rf output
        parameters: 0, 1, OFF, ON"""
        self.write(":OUTPut:STATe {}".format(state))

    def set_frequency(self, freq=2.0):
        """set frequency in GHz"""
        self.write(":SOURce:FREQuency:CW {:f} GHz".format(freq))

    def set_power(self, power=-30):
        """set power in dBm
        the manual says range: -20 to 25, increment: 0.01, *RST -10"""

        self.write(":SOURce:POWer {}dBm".format(power))



# Controlling the santec tsl 5xx laser
# Copyright (c) 2019 Prashanta Kharel, HyperLight Corporation
import time

import pyvisa


class SantecTSLBase:
    # Base class for controlling Santec TSL laser
    def __init__(self, rsc_name=None):
        self.rsc_name = rsc_name
        self.wav_min = None
        self.wav_max = None

    def initialize(self):
        pass

    def getPowerState(self):
        # return 0 for OFF, 1 for ON
        pass

    def turnLaserDiodeON(self, onoroff):
        # onoroff: True means ON, False means OFF
        pass

    def setAttenuation(self, valuedB):
        pass

    def setWavelength(self, valueWavlength):
        pass

    def setFine(self, valueFine):
        pass

    def openShutter(self, onoroff):
        # onoroff: True means open, False means close
        pass

    def sweepSettings(self, numOfSweeps, triggerOut=True):
        pass

    def setSweepCycles(self, numOfSweeps):
        pass

    def setTriggerOut(self, triggerOut=True):
        pass

    def startSweep(self):
        pass

    def startSweepLoop(self):
        pass

    def stopSweep(self):
        pass

    def setSweepSpeed(self, sweepSpeed_nm):
        pass

    def setStartWavelength(self, startWavelength_nm):
        pass

    def setStopWavelength(self, stopWavelength_nm):
        pass

    def setWavelengthUnit(self, unit):
        pass

    def getInternalPower(self):
        pass

    def close(self):
        pass

    def setSweepMode(self, nmode):
        pass

    def setSweepStep(self, step_nm):
        pass


class SantecTSL(SantecTSLBase):
    def __init__(self, rsc_name=None):
        self.rsc_name = rsc_name
        rm = pyvisa.ResourceManager()
        if rsc_name is not None:
            self.rsc_name = rsc_name
        self.rsc = rm.open_resource(self.rsc_name)
        print("IDN: " + self.rsc.query("*IDN?"))
        # wavelength limits
        self.wav_min = None
        self.wav_max = None

    def initialize(self):
        self.setWavelengthUnit("nm")
        time.sleep(0.3)
        self.openShutter(False)
        self.wav_min, self.wav_max = self.getWavlengthLimits()
        # check to power state and turn it ON if it is OFF
        if self.getPowerState() == 0:
            self.turnLaserDiodeON(True)
            print("Waiting for laser to warm up...", end="")
            while not self.getPowerState():
                print(".", end="")
                time.sleep(1)
            print("Done!")
        self.setAttenuation(30)
        time.sleep(0.3)
        self.openShutter(1)  # open the shutter 1: shutter is on
        print(
            "min wavelength: {} nm, max wavelength: {} nm".format(
                self.wav_min, self.wav_max
            )
        )

    def checkWavelength(self, wavelength):
        if not (self.wav_min <= wavelength <= self.wav_max):
            error_msg = "Wavelength {} is outside my range".format(wavelength)
            raise Exception(error_msg)

    def getPowerState(self):
        # return 0 for OFF, 1 for ON
        return int(self.rsc.query(":POWer:STATe?"))

    def turnLaserDiodeON(self, onoroff):
        # onoroff: True means ON, False means OFF
        if onoroff:
            self.rsc.write(":POWer:STATe 1")
        else:
            self.rsc.write(":POWer:STATe 0")

    def setAttenuation(self, valuedB):
        self.rsc.write(":POW:ATT " + str(valuedB))

    def setWavelength(self, valueWavlength):
        self.checkWavelength(valueWavlength)
        self.rsc.write(":WAVelength " + str(valueWavlength))

    def setFine(self, valueFine):
        if -100.0 < valueFine < 100.0:
            self.rsc.write("WAVelength:FINe {:.2f}".format(valueFine))
        else:
            print("Fine tuning value {:.3} is out of range".format(float(valueFine)))

    def openShutter(self, onoroff):
        # onoroff: True means open, False means close
        if onoroff:
            self.rsc.write(":POW:SHUT 0")
            print("Shutter is now open!")
        else:
            self.rsc.write(":POW:SHUT 1")
            print("Shutter is now closed!")

    def sweepSettings(self, numOfSweeps, triggerOut=True):
        # write sweep settings to laser. numOfSweeps=0 for continous
        self.rsc.write(":WAVelength:SWEep:MODe 1")  # sweep mode: 1way continuous
        time.sleep(0.3)
        # use cycles if Range 0~999. 0 means infinite sweep
        self.rsc.write(":WAVelength:SWEep:CYCLes " + str(numOfSweeps))
        if numOfSweeps == 0:
            self.rsc.write(":WAV:SWE:REP")  # continuously repeat the sweeps
        time.sleep(0.3)
        if triggerOut:
            # sends a trigger out at the start of a sweep
            self.rsc.write(":TRIGger: OUTPut 2")
        time.sleep(0.5)
        # self.startSweep()       # start the sweep

    def setSweepCycles(self, numOfSweeps):
        self.rsc.write(
            ":WAVelength:SWEep:CYCLes " + str(numOfSweeps)
        )  # if Range 0~999. 0 means infinite sweep
        if numOfSweeps == 0:
            self.rsc.write(":WAV:SWE:REP")  # continuously repeat the sweeps

    def setTriggerOut(self, triggerOut=True):
        if triggerOut:
            # sends a trigger out at the start of a sweep
            self.rsc.write(":TRIGger: OUTPut 2")
        else:
            self.rsc.write(":TRIGger: OUTPut 0")

    def startSweep(self):
        # start the wavelength sweep
        # time.sleep(0.5)
        self.rsc.write(":WAVelength:SWEep 1")

    def startSweepLoop(self):
        self.rsc.write(":WAVelength:SWEep:STATe:REPeat")

    def stopSweep(self):
        # stop the wavelength sweep
        # for some reason must first pause, then stop it.
        self.rsc.write(":WAVelength:SWEep 2")
        time.sleep(0.1)
        self.rsc.write(":WAVelength:SWEep 0")

    def setSweepSpeed(self, sweepSpeed_nm):
        self.rsc.write(":WAVelength:SWEep:SPEed " + str(sweepSpeed_nm))

    def setStartWavelength(self, startWavelength_nm):
        # speed_light_vacuum = 299792458
        # startfrequency = 299792458 / startWavelength_nm * 1.0e-3  # in THz
        self.checkWavelength(startWavelength_nm)
        self.rsc.write(":WAVelength:SWEep:STARt %.5f" % startWavelength_nm)

    def setStopWavelength(self, stopWavelength_nm):
        # speed_light_vacuum = 299792458
        # stopfrequency = 299792458 / stopWavelength_nm * 1.0e-3  # in THz
        self.checkWavelength(stopWavelength_nm)
        self.rsc.write(":WAVelength:SWEep:STOP %.5f" % stopWavelength_nm)
        # self.laser.write(':FREQuency:SWEep:STOP %.5f' % stopfrequency)

    def setWavelengthUnit(self, unit):
        if unit == "nm":
            self.rsc.write(":WAVelength:UNIT 0")  # 0 means in nm
        elif unit == "THz" or unit == "thz":
            self.rsc.write(":WAVelength:UNIT 1")  # 1 means in  THz
        else:
            raise Exception("Unrecognized unit")

    def getInternalPower(self):
        return self.rsc.query(":POWer:ACTual?")

    def setSweepMode(self, nmode):
        self.rsc.write("WAVelength:SWEep:MODe " + str(nmode))

    def setSweepStep(self, step_nm):
        self.rsc.write("WAVelength:SWEep:STEp " + str(step_nm))

    def getWavlengthLimits(self):
        wav_min = float(self.rsc.query("WAV:MIN?"))
        wav_max = float(self.rsc.query("WAV:MAX?"))
        return wav_min, wav_max

    def close(self):
        self.stopSweep()  # stop the sweep
        time.sleep(0.3)
        self.setAttenuation(30)
        time.sleep(0.3)
        self.openShutter(0)
        # self.turnLaserDiodeON(0)
        self.rsc.close()


class SantecTSLDummy(SantecTSLBase):
    # Dummy class for controlling Santec TSL laser
    def __init__(self, rsc_name=None):
        super().__init__(rsc_name)
        self.already_on = False

    def initialize(self, rsc_name=None):
        self.wav_min = 1480.0  # nm
        self.wav_max = 1630.0  # nm
        if rsc_name is not None:
            self.rsc_name = rsc_name
        if not self.already_on:
            print("Waiting for dummy TSL to warm up...")
            time.sleep(1)
            print("Warmup complete")

    def getPowerState(self):
        # return 0 for OFF, 1 for ON
        return 1

    def getInternalPower(self):
        return 2.000



import time

import pyvisa


class Pollux(object):
    """Python class for Pollux motor, by Di Zhu 2016"""

    def __init__(self, visa_name, baud_rate=19200, axis=1):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name, baud_rate=baud_rate)
        self.pyvisa.timeout = 5000  # set response time in milliseconds
        self.axis = str(axis)

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def position(self):
        self.query(
            self.axis + " npos"
        )  # there's a bug here; you have to ask for the position twice to get the real result
        print("position = " + str(float(self.query(self.axis + " npos"))))

    def status(self):
        return int(self.query(self.axis + " nstatus"))
        # 1 means moving in progress

    def reset(self):
        self.write("*RST")
        self.write(self.axis + " ncal")
        # self.write(self.axis + ' setnpos')
        while self.status() == 1:
            print("moving...")
            time.sleep(0.5)  # wait time in seconds
        self.position()

    # def move_with_speed(self, speed=1.0):
    #     #mm/s; range: +-2000
    #     #this starts a moving with a constant speed
    #     self.write(str(speed) + ' ' + self.axis + ' speed')

    def move_to(self, coordinate):
        self.write(str(float(coordinate)) + " " + self.axis + " nm")
        while self.status() == 1:
            print("moving...")
            time.sleep(0.5)  # wait time in seconds
        self.position()

    def move_by(self, distance):
        self.write(str(float(distance)) + " " + self.axis + " nr")
        while self.status() == 1:
            print("moving...")
            time.sleep(0.5)  # wait time in seconds
        self.position()



# -*- coding: utf-8 -*-
"""
Created on Mon Jan 23 10:39:04 2023

@author: QNN
"""

import pyvisa

LAKESHORE_MODE_MAP = {
    "0": "Off",
    "1": "Closed Loop PID",
    "2": "Zone",
    "3": "Open Loop",
    "4": "Monitor Out",
    "5": "Warmup Supply",
}


class Lakeshore336(object):
    def __init__(self, visa_name):
        rm = pyvisa.ResourceManager()
        self.pyvisa = rm.open_resource(visa_name)
        self.pyvisa.timeout = 5000  # Set response timeout (in milliseconds)
        # self.pyvisa.query_delay = 1 # Set extra delay time between write and read commands

    def read(self):
        return self.pyvisa.read()

    def write(self, string):
        self.pyvisa.write(string)

    def query(self, string):
        return self.pyvisa.query(string)

    def reset(self):
        self.write("*RST")

    def read_temp(self, channel="A"):
        self.write("KRDG? " + channel + "[term]")  # In form of ":INPUT? A:TEMP", A-D
        t = self.read()
        temperature = float(t[1:8])
        return temperature

    def set_setpoint(self, setpoint=295, output=1, ramp=0, rate_value=10):
        self.write("SETP " + str(output) + "," + str(setpoint))

    def set_ramp(self, output=1, on=1, rate_value=0.1):
        self.write("RAMP " + str(output) + "," + str(on) + "," + str(rate_value))

    def set_range(self, output=1, range_set=1):
        self.write("RANGE " + str(output) + "," + str(range_set))

    def set_output(self, output=1, mode=0, input_sensor=4, enable=0):
        """Specifies the control mode. Valid entries: 0 = Off, 1 = Closed
        Loop PID, 2 = Zone, 3 = Open Loop, 4 = Monitor Out,
        5 = Warmup Supply"""
        self.write(
            "OUTMODE "
            + str(output)
            + ","
            + str(mode)
            + ","
            + str(input_sensor)
            + ","
            + str(enable)
        )

    def get_setpoint(self, output=1):
        s = self.query("SETP? " + str(output))
        return float(s)

    def get_range(self, output=1):
        s = self.query("RANGE? " + str(output))
        return s

    def get_ramp(self, output=1):
        s = self.query("RAMP? " + str(output))
        return s

    def get_output(self, output=1):
        s = self.query("OUTMODE? " + str(output))
        output = s.split(",")
        MODE = str(output[0])
        INPUT = str(output[1])
        ENABLE = str(output[2])
        return s

    def get_formatted_output(self, output=1):
        s = self.query("OUTMODE? " + str(output))
        output = s.split(",")
        MODE = str(output[0])
        INPUT = str(output[1])
        ENABLE = str(output[2])
        print(
            "MODE: "
            + LAKESHORE_MODE_MAP[MODE]
            + "\n INPUT: "
            + INPUT
            + "\n ENABLE: "
            + str(bool(ENABLE))
        )
        return s

    def all_off(self):
        self.set_range(1, 0)
        self.set_range(2, 0)
        self.set_range(3, 0)
        self.set_range(4, 0)
